searchData={"items":[{"type":"module","title":"LoggerHandlerKit.Act","doc":"Functions that trigger log events.\n\nThe main purpose of these functions is to serve as inspiration and examples of\ntest cases. They should be usable directly, but users are expected to go beyond them\nsooner rather than later. This is also the reason why they are not customizable\nand not DRY. Ultimately, you should be able to understand what's going on by looking at\nthe source code with minimal jumping throughout the code base.\n\nEach function represents a case of interest, potentially with different flavors to \nhighlight various cases.\n\nThe functions are divided into four groups: Basic, OTP, SASL and Metadata.","ref":"LoggerHandlerKit.Act.html"},{"type":"module","title":"Basic - LoggerHandlerKit.Act","doc":"These cases focus on log messages a logger handler might encounter. For all cases \nhere, the logger handler will execute in the same process as the test. The complexity \ncomes from the diversity of data types. It's a good idea to make sure your logger \nhandler can handle and format all of these.","ref":"LoggerHandlerKit.Act.html#module-basic"},{"type":"module","title":"OTP - LoggerHandlerKit.Act","doc":"These cases focus on reports that become relevant once OTP is involved. Consequently, the \nprocess where logger handler is called is likely different from the test process, which \nmakes test setup more involved and makes ownership important. This is also when Logger \nTranslator becomes relevant, as a lot of OTP errors originate in Erlang and Elixir translates \nthem to look more familiar.\n\nMost of the time, the flow of these logs is governed by the `handle_otp_reports` [Logger\noption](`Logger#module-boot-configuration`) (enabled by default).\n\nA lot of handlers choose to give GenServer errors special treatment, so these \ncases are a good way to generate some samples.","ref":"LoggerHandlerKit.Act.html#module-otp"},{"type":"module","title":"SASL - LoggerHandlerKit.Act","doc":"What are SASL logs? Whatever they were, they aren't anymore. [The concept was\ndeprecated](https://www.erlang.org/doc/apps/sasl/error_logging.html) in Erlang/OTP 21.\nSo for Elixir, SASL logs are the logs that will be skipped if the `handle_sasl_reports`\nconfiguration option is set to `false`, which are all logs which domain matches `[:otp, :sasl | _]`\npattern.\n\nIn real life, SASL logs look like reports from supervisors about things that you \nwould expect: child process restarts and such. They are skipped by Elixir by \ndefault, but a thorough handler might have an interest in them.","ref":"LoggerHandlerKit.Act.html#module-sasl"},{"type":"module","title":"Metadata - LoggerHandlerKit.Act","doc":"These cases focus on challenges that arise from metadata.","ref":"LoggerHandlerKit.Act.html#module-metadata"},{"type":"function","title":"LoggerHandlerKit.Act.bare_process_crash/2","doc":"Bare process crashes are reported as simple string errors, not reports. Another fun\nfact is that an exit, even for a weird reason, is not an error and is not logged.\n  \n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#bare_process_crash/2"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.bare_process_crash/2","doc":"```elixir\ntest \"bare process crash exception\", %{handler_id: handler_id, handler_ref: ref} do\n  LoggerHandlerKit.Act.bare_process_crash(handler_id, :exception)\n  LoggerHandlerKit.Assert.assert_logged(ref)\n\n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#bare_process_crash/2-example-test"},{"type":"function","title":"Example Log Event - LoggerHandlerKit.Act.bare_process_crash/2","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{emulator: true, tag: :error},\n    pid: #PID<0.214.0>,\n    time: 1745101176502155,\n    gl: #PID<0.69.0>,\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-bare_process_crash/1-fun-0-\", 0,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 1106,\n          error_info: %{module: Exception}\n        ]}\n     ]}\n  },\n  msg: {:string,\n   [\n     \"Process \",\n     \"#PID<0.214.0>\",\n     \" raised an exception\",\n     10,\n     \"** (RuntimeError) oops\",\n     [\"\n    \" |\n      \"(logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:1106: anonymous fn/0 in LoggerHandlerKit.Act.bare_process_crash/1\"]\n   ]},\n  level: :error\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#bare_process_crash/2-example-log-event"},{"type":"function","title":"LoggerHandlerKit.Act.chardata_message/1","doc":"[Chardata](`m:IO#module-chardata`) is another tricky type that can be passed directly to the `Logger`.\n\nChardata is an arbitrarily nested and potentially **improper** list.\n\n```elixir\nLogger.info([?H, [\"ello\", []], 32 | ~c\"World\"])\n```\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#chardata_message/1"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.chardata_message/1","doc":"```elixir\ntest \"improper chardata message\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.chardata_message(:improper)\n  LoggerHandlerKit.Assert.assert_logged(ref)\n  \n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#chardata_message/1-example-test"},{"type":"function","title":"Example Log Event - LoggerHandlerKit.Act.chardata_message/1","doc":"```elixir\n%{\n  meta: %{\n    pid: #PID<0.205.0>,\n    time: 1744681993743465,\n    gl: #PID<0.69.0>,\n    domain: [:elixir]\n  },\n  msg: {:string, [72, [\"ello\", []], 32, 87, 111, 114, 108, 100]},\n  level: :info\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#chardata_message/1-example-log-event"},{"type":"function","title":"LoggerHandlerKit.Act.charlist_message/0","doc":"[Charlists](`e:elixir:binaries-strings-and-charlists.html#charlists`) can be passed to `Logger` functions:\n\n```elixir\nLogger.info(~c\"Hello world\")\n```\n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#charlist_message/0"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.charlist_message/0","doc":"```elixir\ntest \"charlist message\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.charlist_message()\n  LoggerHandlerKit.Assert.assert_logged(ref)\n  \n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#charlist_message/0-example-test"},{"type":"function","title":"Example Log Event - LoggerHandlerKit.Act.charlist_message/0","doc":"```elixir\n%{\n  meta: %{\n    pid: #PID<0.205.0>,\n    time: 1744680216246655,\n    gl: #PID<0.69.0>,\n    domain: [:elixir]\n  },\n  msg: {:string, ~c\"Hello World\"},\n  level: :info\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#charlist_message/0-example-log-event"},{"type":"function","title":"LoggerHandlerKit.Act.gen_statem_crash/1","doc":"`:gen_statem` crashes are similar to `GenServer`\n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#gen_statem_crash/1"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.gen_statem_crash/1","doc":"```elixir\ntest \"gen_statem crash exception\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.gen_statem_crash()\n  LoggerHandlerKit.Assert.assert_logged(ref)\n\n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#gen_statem_crash/1-example-test"},{"type":"function","title":"Example Log Event (< 1.19) - LoggerHandlerKit.Act.gen_statem_crash/1","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error, report_cb: &:gen_statem.format_log/1},\n    line: 4975,\n    pid: #PID<0.321.0>,\n    time: 1745097781898702,\n    file: ~c\"gen_statem.erl\",\n    gl: #PID<0.69.0>,\n    domain: [:otp],\n    report_cb: &:gen_statem.format_log/2,\n    mfa: {:gen_statem, :error_info, 7},\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-gen_statem_crash/0-fun-0-\", 0,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 875,\n          error_info: %{module: Exception}\n        ]},\n       {:gen_statem, :loop_state_callback, 11,\n        [file: ~c\"gen_statem.erl\", line: 3735]},\n       {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n     ]}\n  },\n  msg: {:string,\n   [\n     [\n       \":gen_statem \",\n       \"#PID<0.321.0>\",\n       \" terminating\",\n       [\n         [10 | \"** (RuntimeError) oops\"],\n         [\"\n    \" |\n          \"(logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:875: anonymous fn/0 in LoggerHandlerKit.Act.gen_statem_crash/0\"],\n         [\"\n    \" |\n          \"(stdlib 6.2.1) gen_statem.erl:3735: :gen_statem.loop_state_callback/11\"],\n         [\"\n    \" |\n          \"(stdlib 6.2.1) proc_lib.erl:329: :proc_lib.init_p_do_apply/3\"]\n       ],\n       [],\n       \"\nQueue: [{{:call, {#PID<0.312.0>, #Reference<0.4279064375.3240624134.253706>}}, {:run, #Function<0.101033180/0 in LoggerHandlerKit.Act.gen_statem_crash/0>}}]\",\n       \"\nPostponed: []\"\n     ],\n     \"\nState: \",\n     \"{:started, nil}\",\n     \"\nCallback mode: \",\n     \":state_functions, state_enter: false\",\n     \"\nClient \",\n     \"#PID<0.312.0>\",\n     \" is alive\n\",\n     [\"\n    \" | \"(stdlib 6.2.1) gen.erl:241: :gen.do_call/4\"],\n     [\"\n    \" | \"(stdlib 6.2.1) gen_statem.erl:3250: :gen_statem.call/3\"],\n     [\"\n    \" |\n      \"(logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:871: LoggerHandlerKit.Act.gen_statem_crash/0\"],\n     [\"\n    \" |\n      \"test/default_logger_test.exs:243: LoggerHandlerKit.DefaultLoggerTest.\"test Advanced gen_statem crash exception\"/1\"],\n     [\"\n    \" |\n      \"(ex_unit 1.18.3) lib/ex_unit/runner.ex:511: ExUnit.Runner.exec_test/2\"],\n     [\"\n    \" |\n      \"(ex_unit 1.18.3) lib/ex_unit/capture_log.ex:113: ExUnit.CaptureLog.with_log/2\"],\n     [\"\n    \" |\n      \"(ex_unit 1.18.3) lib/ex_unit/runner.ex:460: anonymous fn/3 in ExUnit.Runner.maybe_capture_log/3\"],\n     [\"\n    \" | \"(stdlib 6.2.1) timer.erl:595: :timer.tc/2\"],\n     [\"\n    \" |\n      \"(ex_unit 1.18.3) lib/ex_unit/runner.ex:433: anonymous fn/6 in ExUnit.Runner.spawn_test_monitor/4\"]\n   ]},\n  level: :error\n}\n```","ref":"LoggerHandlerKit.Act.html#gen_statem_crash/1-example-log-event-1-19"},{"type":"function","title":"Example Log Event (1.19+) - LoggerHandlerKit.Act.gen_statem_crash/1","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error, report_cb: &:gen_statem.format_log/1},\n    line: 4975,\n    pid: #PID<0.222.0>,\n    time: 1745097734784539,\n    file: ~c\"gen_statem.erl\",\n    gl: #PID<0.69.0>,\n    domain: [:otp],\n    report_cb: &Logger.Utils.translated_cb/1,\n    mfa: {:gen_statem, :error_info, 7},\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-gen_statem_crash/0-fun-0-\", 0,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 843,\n          error_info: %{module: Exception}\n        ]},\n       {:gen_statem, :loop_state_callback, 11,\n        [file: ~c\"gen_statem.erl\", line: 3735]},\n       {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n     ]}\n  },\n  msg: {:report,\n   %{\n     label: {:gen_statem, :terminate},\n     name: #PID<0.222.0>,\n     reason: {:error, %RuntimeError{message: \"oops\"},\n      [\n        {LoggerHandlerKit.Act, :\"-gen_statem_crash/0-fun-0-\", 0,\n         [\n           file: ~c\"lib/logger_handler_kit/act.ex\",\n           line: 843,\n           error_info: %{module: Exception}\n         ]},\n        {:gen_statem, :loop_state_callback, 11,\n         [file: ~c\"gen_statem.erl\", line: 3735]},\n        {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n      ]},\n     log: [],\n     state: {:started, nil},\n     queue: [\n       {{:call, {#PID<0.211.0>, #Reference<0.1625241344.3509321731.89483>}},\n        {:run,\n         #Function<0.101033180/0 in LoggerHandlerKit.Act.gen_statem_crash/0>}}\n     ],\n     modules: [LoggerHandlerKit.GenStatem],\n     process_label: :undefined,\n     client_info: {#PID<0.211.0>,\n      {#PID<0.211.0>,\n       [\n         {:gen, :do_call, 4, [file: ~c\"gen.erl\", line: 241]},\n         {:gen_statem, :call, 3, [file: ~c\"gen_statem.erl\", line: 3250]},\n         {LoggerHandlerKit.Act, :gen_statem_crash, 0,\n          [file: ~c\"lib/logger_handler_kit/act.ex\", line: 839]},\n         {LoggerHandlerKit.DefaultLoggerTest,\n          :\"test Advanced gen_statem crash exception\", 1,\n          [file: ~c\"test/default_logger_test.exs\", line: 243]},\n         {ExUnit.Runner, :exec_test, 2,\n          [file: ~c\"lib/ex_unit/runner.ex\", line: 515]},\n         {ExUnit.CaptureLog, :with_log, 2,\n          [file: ~c\"lib/ex_unit/capture_log.ex\", line: 117]},\n         {ExUnit.Runner, :\"-maybe_capture_log/3-fun-0-\", 3,\n          [file: ~c\"lib/ex_unit/runner.ex\", line: 464]},\n         {:timer, :tc, 2, [file: ~c\"timer.erl\", line: 595]},\n         {ExUnit.Runner, :\"-spawn_test_monitor/4-fun-1-\", 6,\n          [file: ~c\"lib/ex_unit/runner.ex\", line: 437]}\n       ]}},\n     callback_mode: :state_functions,\n     postponed: [],\n     timeouts: {0, []},\n     state_enter: false,\n     elixir_translation: [...]\n     ]\n   }},\n  level: :error\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#gen_statem_crash/1-example-log-event-1-19"},{"type":"function","title":"LoggerHandlerKit.Act.genserver_crash/1","doc":"`GenServer` crash is a very common error message. So common, in fact, that a lot of \nhandlers put additional effort into extracting useful information from GenServer reports, \nsuch as process name, labels, and client information. Before Elixir 1.19, that was \npretty hard to do as `Logger.Translator` would replace the structured report with an \nunstructured string, but with 1.19 the reports survive translation.\n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#genserver_crash/1"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.genserver_crash/1","doc":"```elixir\ntest \"genserver crash exception\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.genserver_crash(:exception)\n  LoggerHandlerKit.Assert.assert_logged(ref)\n  \n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#genserver_crash/1-example-test"},{"type":"function","title":"Example Log Event (< 1.19) - LoggerHandlerKit.Act.genserver_crash/1","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error, report_cb: &:gen_server.format_log/1},\n    line: 2646,\n    pid: #PID<0.316.0>,\n    time: 1744685313255062,\n    file: ~c\"gen_server.erl\",\n    gl: #PID<0.69.0>,\n    domain: [:otp],\n    mfa: {:gen_server, :error_info, 8},\n    report_cb: &:gen_server.format_log/2,\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-genserver_crash/0-fun-0-\", 0,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 319,\n          error_info: %{module: Exception}\n        ]},\n       {:gen_server, :try_handle_call, 4,\n        [file: ~c\"gen_server.erl\", line: 2381]},\n       {:gen_server, :handle_msg, 6, [file: ~c\"gen_server.erl\", line: 2410]},\n       {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n     ]}\n  },\n  msg: {:string,\n   [\n     [\n       \"GenServer \",\n       \"#PID<0.316.0>\",\n       \" terminating\",\n       [\n         [10 | \"** (RuntimeError) oops\"],\n         [\"\n    \" |\n          \"(logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:319: anonymous fn/0 in LoggerHandlerKit.Act.genserver_crash/0\"],\n         [\"\n    \" |\n          \"(stdlib 6.2.1) gen_server.erl:2381: :gen_server.try_handle_call/4\"],\n         [\"\n    \" |\n          \"(stdlib 6.2.1) gen_server.erl:2410: :gen_server.handle_msg/6\"],\n         [\"\n    \" |\n          \"(stdlib 6.2.1) proc_lib.erl:329: :proc_lib.init_p_do_apply/3\"]\n       ],\n       [],\n       \"\nLast message\",\n       [\" (from \", \"#PID<0.307.0>\", \")\"],\n       \": \",\n       \"{:run, #Function<0.66655897/0 in LoggerHandlerKit.Act.genserver_crash/0>}\"\n     ],\n     \"\nState: \",\n     \"nil\",\n     \"\nClient \",\n     \"#PID<0.307.0>\",\n     \" is alive\n\",\n     [\"\n    \" | \"(stdlib 6.2.1) gen.erl:260: :gen.do_call/4\"],\n     [\"\n    \" | \"(elixir 1.18.3) lib/gen_server.ex:1125: GenServer.call/3\"],\n     [\"\n    \" |\n      \"(logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:319: LoggerHandlerKit.Act.genserver_crash/0\"],\n     [\"\n    \" |\n      \"test/default_logger_test.exs:85: LoggerHandlerKit.DefaultLoggerTest.\"test Advanced genserver crash\"/1\"],\n     [\"\n    \" |\n      \"(ex_unit 1.18.3) lib/ex_unit/runner.ex:511: ExUnit.Runner.exec_test/2\"],\n     [\"\n    \" |\n      \"(ex_unit 1.18.3) lib/ex_unit/capture_log.ex:113: ExUnit.CaptureLog.with_log/2\"],\n     [\"\n    \" |\n      \"(ex_unit 1.18.3) lib/ex_unit/runner.ex:460: anonymous fn/3 in ExUnit.Runner.maybe_capture_log/3\"],\n     [\"\n    \" | \"(stdlib 6.2.1) timer.erl:595: :timer.tc/2\"],\n     [\"\n    \" |\n      \"(ex_unit 1.18.3) lib/ex_unit/runner.ex:433: anonymous fn/6 in ExUnit.Runner.spawn_test_monitor/4\"]\n   ]},\n  level: :error\n}\n```","ref":"LoggerHandlerKit.Act.html#genserver_crash/1-example-log-event-1-19"},{"type":"function","title":"Example Log Event (1.19+) - LoggerHandlerKit.Act.genserver_crash/1","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error, report_cb: &:gen_server.format_log/1},\n    line: 2646,\n    pid: #PID<0.203.0>,\n    time: 1744684762852118,\n    file: ~c\"gen_server.erl\",\n    gl: #PID<0.69.0>,\n    domain: [:otp],\n    report_cb: &Logger.Utils.translated_cb/1,\n    mfa: {:gen_server, :error_info, 8},\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-genserver_crash/0-fun-0-\", 0,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 246,\n          error_info: %{module: Exception}\n        ]},\n       {:gen_server, :try_handle_call, 4,\n        [file: ~c\"gen_server.erl\", line: 2381]},\n       {:gen_server, :handle_msg, 6, [file: ~c\"gen_server.erl\", line: 2410]},\n       {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n     ]}\n  },\n  msg: {:report,\n   %{\n     label: {:gen_server, :terminate},\n     name: #PID<0.203.0>,\n     reason: {%RuntimeError{message: \"oops\"},\n      [\n        {LoggerHandlerKit.Act, :\"-genserver_crash/0-fun-0-\", 0,\n         [\n           file: ~c\"lib/logger_handler_kit/act.ex\",\n           line: 246,\n           error_info: %{module: Exception}\n         ]},\n        {:gen_server, :try_handle_call, 4,\n         [file: ~c\"gen_server.erl\", line: 2381]},\n        {:gen_server, :handle_msg, 6, [file: ~c\"gen_server.erl\", line: 2410]},\n        {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n      ]},\n     log: [],\n     state: nil,\n     client_info: {#PID<0.192.0>,\n      {#PID<0.192.0>,\n       [\n         {:gen, :do_call, 4, [file: ~c\"gen.erl\", line: 260]},\n         {GenServer, :call, 3, [file: ~c\"lib/gen_server.ex\", line: 1139]},\n         {LoggerHandlerKit.Act, :genserver_crash, 0,\n          [file: ~c\"lib/logger_handler_kit/act.ex\", line: 246]},\n         {LoggerHandlerKit.DefaultLoggerTest, :\"test Advanced genserver crash\",\n          1, [file: ~c\"test/default_logger_test.exs\", line: 85]},\n         {ExUnit.Runner, :exec_test, 2,\n          [file: ~c\"lib/ex_unit/runner.ex\", line: 515]},\n         {ExUnit.CaptureLog, :with_log, 2,\n          [file: ~c\"lib/ex_unit/capture_log.ex\", line: 117]},\n         {ExUnit.Runner, :\"-maybe_capture_log/3-fun-0-\", 3,\n          [file: ~c\"lib/ex_unit/runner.ex\", line: 464]},\n         {:timer, :tc, 2, [file: ~c\"timer.erl\", line: 595]},\n         {ExUnit.Runner, :\"-spawn_test_monitor/4-fun-1-\", 6,\n          [file: ~c\"lib/ex_unit/runner.ex\", line: 437]}\n       ]}},\n     last_message: {:run,\n      #Function<0.66655897/0 in LoggerHandlerKit.Act.genserver_crash/0>},\n     process_label: :undefined,\n     elixir_translation: [ ... ]\n   }},\n  level: :error\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#genserver_crash/1-example-log-event-1-19"},{"type":"function","title":"LoggerHandlerKit.Act.genserver_init_crash/0","doc":"If `c:GenServer.init/1` callback fails, the caller receives an error tuple. Seemingly, nothing \n  exceptional happened... but SASL knows. And will report.\n\n```elixir\ndefmodule MyGenserver do\n  use GenServer\n\n  def init(_), do: raise \"oops\"\nend\n\n{:error, {%RuntimeError{}, _}} = GenServer.start(MyGenserver, nil)\n\n# This log is only sent if handle_sasl_reports is set to true\n# 21:54:45.734 [error] Process #PID<0.115.0> terminating\n# ** (RuntimeError) oops\n#     iex:4: MyGenserver.init/1\n#     (stdlib 6.2.1) gen_server.erl:2229: :gen_server.init_it/2\n#     (stdlib 6.2.1) gen_server.erl:2184: :gen_server.init_it/6\n#     (stdlib 6.2.1) proc_lib.erl:329: :proc_lib.init_p_do_apply/3\n# Initial Call: MyGenserver.init/1\n# Ancestors: [#PID<0.108.0>, #PID<0.98.0>]\n# Message Queue Length: 0\n# Messages: []\n# Links: []\n# Dictionary: []\n# Trapping Exits: false\n# Status: :running\n# Heap Size: 233\n# Stack Size: 29\n# Reductions: 57\n```\n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#genserver_init_crash/0"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.genserver_init_crash/0","doc":"```elixir\ntest \"genserver init crash\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.genserver_init_crash()\n  LoggerHandlerKit.Assert.assert_logged(ref)\n\n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#genserver_init_crash/0-example-test"},{"type":"function","title":"Example Log Event (< 1.19) - LoggerHandlerKit.Act.genserver_init_crash/0","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error_report, type: :crash_report},\n    initial_call: {LoggerHandlerKit.GenServer, :init, 1},\n    line: 948,\n    pid: #PID<0.214.0>,\n    time: 1744844942033945,\n    file: ~c\"proc_lib.erl\",\n    gl: #PID<0.69.0>,\n    domain: [:otp, :sasl],\n    logger_formatter: %{title: ~c\"CRASH REPORT\"},\n    mfa: {:proc_lib, :crash_report, 4},\n    report_cb: &:proc_lib.report_cb/2,\n    ancestors: [#PID<0.205.0>],\n    callers: [#PID<0.205.0>],\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-genserver_init_crash/0-fun-0-\", 0,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 840,\n          error_info: %{module: Exception}\n        ]},\n       {:gen_server, :init_it, 2, [file: ~c\"gen_server.erl\", line: 2229]},\n       {:gen_server, :init_it, 6, [file: ~c\"gen_server.erl\", line: 2184]},\n       {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n     ]}\n  },\n  msg: {:string,\n   [\n     \"Process \",\n     \"#PID<0.214.0>\",\n     \" terminating\",\n     [\n       10,\n       \"** (RuntimeError) oops\",\n       [\"\n    \" |\n        \"(logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:840: anonymous fn/0 in LoggerHandlerKit.Act.genserver_init_crash/0\"],\n       [\"\n    \" | \"(stdlib 6.2.1) gen_server.erl:2229: :gen_server.init_it/2\"],\n       [\"\n    \" | \"(stdlib 6.2.1) gen_server.erl:2184: :gen_server.init_it/6\"],\n       [\"\n    \" |\n        \"(stdlib 6.2.1) proc_lib.erl:329: :proc_lib.init_p_do_apply/3\"]\n     ],\n     [\n       ~c\"\n\",\n       \"Initial Call: \",\n       \"LoggerHandlerKit.GenServer.init/1\",\n       ~c\"\n\",\n       \"Ancestors: \",\n       \"[#PID<0.205.0>]\",\n       [~c\"\n\", \"Message Queue Length\", 58, 32, \"0\"],\n       [~c\"\n\", \"Messages\", 58, 32, \"[]\"],\n       [~c\"\n\", \"Links\", 58, 32, \"[]\"],\n       [~c\"\n\", \"Dictionary\", 58, 32, \"[\"$callers\": [#PID<0.205.0>]]\"],\n       [~c\"\n\", \"Trapping Exits\", 58, 32, \"false\"],\n       [~c\"\n\", \"Status\", 58, 32, \":running\"],\n       [~c\"\n\", \"Heap Size\", 58, 32, \"233\"],\n       [~c\"\n\", \"Stack Size\", 58, 32, \"29\"],\n       [~c\"\n\", \"Reductions\", 58, 32, \"72\"]\n     ],\n     []\n   ]},\n  level: :error\n}\n```","ref":"LoggerHandlerKit.Act.html#genserver_init_crash/0-example-log-event-1-19"},{"type":"function","title":"Example Log Event (1.19+) - LoggerHandlerKit.Act.genserver_init_crash/0","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error_report, type: :crash_report},\n    initial_call: {LoggerHandlerKit.GenServer, :init, 1},\n    line: 948,\n    pid: #PID<0.326.0>,\n    time: 1744844985798744,\n    file: ~c\"proc_lib.erl\",\n    gl: #PID<0.69.0>,\n    domain: [:otp, :sasl],\n    logger_formatter: %{title: ~c\"CRASH REPORT\"},\n    mfa: {:proc_lib, :crash_report, 4},\n    report_cb: &Logger.Utils.translated_cb/1,\n    ancestors: [#PID<0.315.0>],\n    callers: [#PID<0.315.0>],\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-genserver_init_crash/0-fun-0-\", 0,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 868,\n          error_info: %{module: Exception}\n        ]},\n       {:gen_server, :init_it, 2, [file: ~c\"gen_server.erl\", line: 2229]},\n       {:gen_server, :init_it, 6, [file: ~c\"gen_server.erl\", line: 2184]},\n       {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n     ]}\n  },\n  msg: {:report,\n   %{\n     label: {:proc_lib, :crash},\n     report: [\n       [\n         initial_call: {LoggerHandlerKit.GenServer, :init, [:Argument__1]},\n         pid: #PID<0.326.0>,\n         registered_name: [],\n         process_label: :undefined,\n         error_info: {:error, %RuntimeError{message: \"oops\"},\n          [\n            {LoggerHandlerKit.Act, :\"-genserver_init_crash/0-fun-0-\", 0,\n             [\n               file: ~c\"lib/logger_handler_kit/act.ex\",\n               line: 868,\n               error_info: %{module: Exception}\n             ]},\n            {:gen_server, :init_it, 2, [file: ~c\"gen_server.erl\", line: 2229]},\n            {:gen_server, :init_it, 6, [file: ~c\"gen_server.erl\", line: 2184]},\n            {:proc_lib, :init_p_do_apply, 3,\n             [file: ~c\"proc_lib.erl\", line: 329]}\n          ]},\n         ancestors: [#PID<0.315.0>],\n         message_queue_len: 0,\n         messages: [],\n         links: [],\n         dictionary: [\"$callers\": [#PID<0.315.0>]],\n         trap_exit: false,\n         status: :running,\n         heap_size: 233,\n         stack_size: 29,\n         reductions: 72\n       ],\n       []\n     ],\n     elixir_translation: [...]\n   }},\n  level: :error\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#genserver_init_crash/0-example-log-event-1-19"},{"type":"function","title":"LoggerHandlerKit.Act.io_format/0","doc":"[IO format with data](https://www.erlang.org/doc/apps/stdlib/io.html#fwrite/3) is \nan exotic type which exists beyond the string or report categorization.\n\nIO format is an Erlang term, and in the Elixir world we almost never \nencounter it. However, no Erlang thing is truly foreign, so handlers must be\nprepared for it.\n\n```elixir\n:logger.info(\"Hello ~ts\", [\"World\"])\n```\n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#io_format/0"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.io_format/0","doc":"```elixir\ntest \"io format\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.io_format()\n  LoggerHandlerKit.Assert.assert_logged(ref)\n\n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#io_format/0-example-test"},{"type":"function","title":"Example Log Event - LoggerHandlerKit.Act.io_format/0","doc":"```elixir\n%{\n  meta: %{pid: #PID<0.204.0>, time: 1744766862567895, gl: #PID<0.69.0>},\n  msg: {\"Hello ~ts\", [\"World\"]},\n  level: :info\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#io_format/0-example-log-event"},{"type":"function","title":"LoggerHandlerKit.Act.keyword_report/0","doc":"[Keyword lists](`Keyword`) can be passed to the `Logger`, and in this case we call it a [report](https://www.erlang.org/doc/apps/kernel/logger_chapter.html#log-message).\n\n```elixir\nLogger.info([hello: \"world\"])\n```\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#keyword_report/0"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.keyword_report/0","doc":"```elixir\ntest \"keyword report\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.keyword_report()\n  LoggerHandlerKit.Assert.assert_logged(ref)\n  \n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#keyword_report/0-example-test"},{"type":"function","title":"Example Log Event - LoggerHandlerKit.Act.keyword_report/0","doc":"```elixir\n%{\n  meta: %{\n    pid: #PID<0.191.0>,\n    time: 1744682416540252,\n    gl: #PID<0.69.0>,\n    domain: [:elixir]\n  },\n  msg: {:report, [hello: \"world\"]},\n  level: :info\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#keyword_report/0-example-log-event"},{"type":"function","title":"LoggerHandlerKit.Act.log_with_crash_reason/1","doc":"Sometimes, errors are handled and reported as unhandled using [`crash_reason` metadata](guides/unhandled.md#crash_reason-metadata).\n\n```elixir\ntry do\n  raise \"oops\"\nrescue\n  exception -> Logger.error(\"Something went wrong\", crash_reason: {exception, __STACKTRACE__})\nend\n```\n\nDevelopers face a tough choice here: on the one hand, they have both error and\nstacktrace that are perfectly formattable with `Exception.format/3`. On the\nother hand, the user provided an explicit log message. \n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#log_with_crash_reason/1"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.log_with_crash_reason/1","doc":"```elixir\ntest \"log with exception as crash reason\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.log_with_crash_reason(:exception)\n  LoggerHandlerKit.Assert.assert_logged(ref)\n\n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#log_with_crash_reason/1-example-test"},{"type":"function","title":"Example Log Event - LoggerHandlerKit.Act.log_with_crash_reason/1","doc":"```elixir\n%{\n  meta: %{\n    pid: #PID<0.204.0>,\n    time: 1749943344608173,\n    gl: #PID<0.69.0>,\n    domain: [:elixir],\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :log_with_crash_reason, 1,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 531,\n          error_info: %{module: Exception}\n        ]},\n       {LoggerHandlerKit.DefaultLoggerTest,\n        :\"test Basic log with crash reason: exception\", 1,\n        [file: ~c\"test/default_logger_test.exs\", line: 97]},\n       {ExUnit.Runner, :exec_test, 2,\n        [file: ~c\"lib/ex_unit/runner.ex\", line: 522]},\n       {ExUnit.CaptureLog, :with_log, 2,\n        [file: ~c\"lib/ex_unit/capture_log.ex\", line: 117]},\n       {ExUnit.Runner, :\"-maybe_capture_log/3-fun-0-\", 3,\n        [file: ~c\"lib/ex_unit/runner.ex\", line: 471]},\n       {:timer, :tc, 2, [file: ~c\"timer.erl\", line: 595]},\n       {ExUnit.Runner, :\"-spawn_test_monitor/4-fun-1-\", 6,\n        [file: ~c\"lib/ex_unit/runner.ex\", line: 444]}\n     ]}\n  },\n  msg: {:string, \"Handled Exception\"},\n  level: :error\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#log_with_crash_reason/1-example-log-event"},{"type":"function","title":"LoggerHandlerKit.Act.map_report/0","doc":"A `Map` can be passed to the `Logger`, and in this case we call it a [report](https://www.erlang.org/doc/apps/kernel/logger_chapter.html#log-message).\n\n```elixir\nLogger.info(%{hello: \"world\"})\n```\n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#map_report/0"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.map_report/0","doc":"```elixir\ntest \"map report\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.map_report()\n  LoggerHandlerKit.Assert.assert_logged(ref)\n  \n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#map_report/0-example-test"},{"type":"function","title":"Example Log Event - LoggerHandlerKit.Act.map_report/0","doc":"```elixir\n%{\n  meta: %{\n    pid: #PID<0.191.0>,\n    time: 1744683131714605,\n    gl: #PID<0.69.0>,\n    domain: [:elixir]\n  },\n  msg: {:report, %{hello: \"world\"}},\n  level: :info\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#map_report/0-example-log-event"},{"type":"function","title":"LoggerHandlerKit.Act.metadata_serialization/1","doc":"Sets the `extra` key in Logger metadata to a sample value of some interesting type.\n\nMetadata can contain arbitrary Elixir terms, and the primary challenge that\nloggers face when exporting it is serialization. There is no universally good\nway to represent an Elixir term as text, so handlers or formatters must make\nhard choices. Some examples:\n\n* Binary strings can contain non-printable characters.\n* Structs by default don't implement the `String.Chars` protocol. When they do, the implementation might be designed for a different purpose than logging.\n* Tuples can be inspected as text but lose their structure (in JSON), or serialized as lists which preserves structure but misleads about the original type.\n* Charlists are indistinguishable from lists in JSON serialization.\n\nThe default text formatter [skips](`Logger.Formatter#module-metadata`) many of these complex cases.","ref":"LoggerHandlerKit.Act.html#metadata_serialization/1"},{"type":"function","title":"LoggerHandlerKit.Act.plug_error/3","doc":"Starts a web server powered by Cowboy or Bandit and sends a request that triggers an error during the Plug pipeline.\n\nSee [Plug integration guide](guides/plug-integration.md) for more details.\n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#plug_error/3"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.plug_error/3","doc":"```elixir\n# You only need to create your own router if you want to test custom plugs that somehow affect logging\ndefmodule MyPlug do\n  use Plug.Router\n  \n  plug MyCustomPlug\n  plug :match\n  plug :dispatch\n  \n  forward \"/\", to: LoggerHandlerKit.Plug\nend\n\ntest \"Bandit: plug exception\", %{handler_ref: ref, io_ref: io_ref} do\n  LoggerHandlerKit.Act.plug_error(:exception, Bandit, MyPlug)\n  LoggerHandlerKit.Assert.assert_logged(ref)\n  \n  # handler-specific assertions\nend  \n```","ref":"LoggerHandlerKit.Act.html#plug_error/3-example-test"},{"type":"function","title":"Example Log Event (Bandit) - LoggerHandlerKit.Act.plug_error/3","doc":"```elixir\n%{\n  meta: %{\n    line: 242,\n    pid: #PID<0.556.0>,\n    time: 1750196815012775,\n    file: ~c\"lib/bandit/pipeline.ex\",\n    gl: #PID<0.69.0>,\n    domain: [:elixir, :bandit],\n    application: :bandit,\n    mfa: {Bandit.Pipeline, :handle_error, 7},\n    plug: {LoggerHandlerKit.Plug, %{test_pid: #PID<0.240.0>}},\n    conn: %Plug.Conn{...},\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Plug, :\"-do_match/4-fun-1-\", 2,\n        [\n          file: ~c\"lib/logger_handler_kit/plug.ex\",\n          line: 8,\n          error_info: %{module: Exception}\n        ]},\n       {LoggerHandlerKit.Plug, :\"-dispatch/2-fun-0-\", 4,\n        [file: ~c\"deps/plug/lib/plug/router.ex\", line: 246]},\n       {:telemetry, :span, 3,\n        [\n          file: ~c\"/Users/user/projects/logger_handler_kit/deps/telemetry/src/telemetry.erl\",\n          line: 324\n        ]},\n       {LoggerHandlerKit.Plug, :dispatch, 2,\n        [file: ~c\"deps/plug/lib/plug/router.ex\", line: 242]},\n       {LoggerHandlerKit.Plug, :plug_builder_call, 2,\n        [file: ~c\"lib/logger_handler_kit/plug.ex\", line: 1]},\n       {Bandit.Pipeline, :call_plug!, 2,\n        [file: ~c\"lib/bandit/pipeline.ex\", line: 131]},\n       {Bandit.Pipeline, :run, 5, [file: ~c\"lib/bandit/pipeline.ex\", line: 42]},\n       {Bandit.HTTP1.Handler, :handle_data, 3,\n        [file: ~c\"lib/bandit/http1/handler.ex\", line: 13]},\n       {Bandit.DelegatingHandler, :handle_data, 3,\n        [file: ~c\"lib/bandit/delegating_handler.ex\", line: 18]},\n       {Bandit.DelegatingHandler, :handle_continue, 2,\n        [file: ~c\"lib/bandit/delegating_handler.ex\", line: 8]},\n       {:gen_server, :try_handle_continue, 3,\n        [file: ~c\"gen_server.erl\", line: 2335]},\n       {:gen_server, :loop, 7, [file: ~c\"gen_server.erl\", line: 2244]},\n       {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n     ]}\n  },\n  msg: {:string,\n   \"** (RuntimeError) oops\n    (logger_handler_kit 0.2.0) lib/logger_handler_kit/plug.ex:8: anonymous fn/2 in LoggerHandlerKit.Plug.do_match/4\n    (logger_handler_kit 0.2.0) deps/plug/lib/plug/router.ex:246: anonymous fn/4 in LoggerHandlerKit.Plug.dispatch/2\n    (telemetry 1.3.0) /Users/user/projects/logger_handler_kit/deps/telemetry/src/telemetry.erl:324: :telemetry.span/3\n    (logger_handler_kit 0.2.0) deps/plug/lib/plug/router.ex:242: LoggerHandlerKit.Plug.dispatch/2\n    (logger_handler_kit 0.2.0) lib/logger_handler_kit/plug.ex:1: LoggerHandlerKit.Plug.plug_builder_call/2\n    (bandit 1.7.0) lib/bandit/pipeline.ex:131: Bandit.Pipeline.call_plug!/2\n    (bandit 1.7.0) lib/bandit/pipeline.ex:42: Bandit.Pipeline.run/5\n    (bandit 1.7.0) lib/bandit/http1/handler.ex:13: Bandit.HTTP1.Handler.handle_data/3\n    (bandit 1.7.0) lib/bandit/delegating_handler.ex:18: Bandit.DelegatingHandler.handle_data/3\n    (bandit 1.7.0) lib/bandit/delegating_handler.ex:8: Bandit.DelegatingHandler.handle_continue/2\n    (stdlib 6.2.2) gen_server.erl:2335: :gen_server.try_handle_continue/3\n    (stdlib 6.2.2) gen_server.erl:2244: :gen_server.loop/7\n    (stdlib 6.2.2) proc_lib.erl:329: :proc_lib.init_p_do_apply/3\n\"},\n  level: :error\n}\n```","ref":"LoggerHandlerKit.Act.html#plug_error/3-example-log-event-bandit"},{"type":"function","title":"Example Log Event (Cowboy) (Elixir 1.19+) - LoggerHandlerKit.Act.plug_error/3","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error},\n    pid: #PID<0.454.0>,\n    time: 1750197653870258,\n    gl: #PID<0.69.0>,\n    domain: [:cowboy],\n    report_cb: &Logger.Utils.translated_cb/1,\n    conn: %Plug.Conn{...},\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Plug, :\"-do_match/4-fun-1-\", 2,\n        [\n          file: ~c\"lib/logger_handler_kit/plug.ex\",\n          line: 8,\n          error_info: %{module: Exception}\n        ]},\n       {LoggerHandlerKit.Plug, :\"-dispatch/2-fun-0-\", 4,\n        [file: ~c\"deps/plug/lib/plug/router.ex\", line: 246]},\n       {:telemetry, :span, 3,\n        [\n          file: ~c\"/Users/user/projects/logger_handler_kit/deps/telemetry/src/telemetry.erl\",\n          line: 324\n        ]},\n       {LoggerHandlerKit.Plug, :dispatch, 2,\n        [file: ~c\"deps/plug/lib/plug/router.ex\", line: 242]},\n       {LoggerHandlerKit.Plug, :plug_builder_call, 2,\n        [file: ~c\"lib/logger_handler_kit/plug.ex\", line: 1]},\n       {Plug.Cowboy.Handler, :init, 2,\n        [file: ~c\"lib/plug/cowboy/handler.ex\", line: 11]},\n       {:cowboy_handler, :execute, 2,\n        [\n          file: ~c\"/Users/user/projects/logger_handler_kit/deps/cowboy/src/cowboy_handler.erl\",\n          line: 37\n        ]},\n       {:cowboy_stream_h, :execute, 3,\n        [\n          file: ~c\"/Users/user/projects/logger_handler_kit/deps/cowboy/src/cowboy_stream_h.erl\",\n          line: 310\n        ]},\n       {:cowboy_stream_h, :request_process, 3,\n        [\n          file: ~c\"/Users/user/projects/logger_handler_kit/deps/cowboy/src/cowboy_stream_h.erl\",\n          line: 299\n        ]},\n       {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n     ]}\n  },\n  msg: {:report,\n   %{\n     args: [\n       LoggerHandlerKit.Plug.HTTP,\n       #PID<0.454.0>,\n       1,\n       #PID<0.455.0>,\n       {{{%RuntimeError{message: \"oops\"},\n          [\n            {LoggerHandlerKit.Plug, :\"-do_match/4-fun-1-\", 2,\n             [\n               file: ~c\"lib/logger_handler_kit/plug.ex\",\n               line: 8,\n               error_info: %{module: Exception}\n             ]},\n            {LoggerHandlerKit.Plug, :\"-dispatch/2-fun-0-\", 4,\n             [file: ~c\"deps/plug/lib/plug/router.ex\", line: 246]},\n            {:telemetry, :span, 3,\n             [\n               file: ~c\"/Users/user/projects/logger_handler_kit/deps/telemetry/src/telemetry.erl\",\n               line: 324\n             ]},\n            {LoggerHandlerKit.Plug, :dispatch, 2,\n             [file: ~c\"deps/plug/lib/plug/router.ex\", line: 242]},\n            {LoggerHandlerKit.Plug, :plug_builder_call, 2,\n             [file: ~c\"lib/logger_handler_kit/plug.ex\", line: 1]},\n            {Plug.Cowboy.Handler, :init, 2,\n             [file: ~c\"lib/plug/cowboy/handler.ex\", line: 11]},\n            {:cowboy_handler, :execute, 2,\n             [\n               file: ~c\"/Users/user/projects/logger_handler_kit/deps/cowboy/src/cowboy_handler.erl\",\n               line: 37\n             ]},\n            {:cowboy_stream_h, :execute, 3,\n             [\n               file: ~c\"/Users/user/projects/logger_handler_kit/deps/cowboy/src/cowboy_stream_h.erl\",\n               line: 310\n             ]},\n            {:cowboy_stream_h, :request_process, 3,\n             [\n               file: ~c\"/Users/user/projects/logger_handler_kit/deps/cowboy/src/cowboy_stream_h.erl\",\n               line: 299\n             ]},\n            {:proc_lib, :init_p_do_apply, 3,\n             [file: ~c\"proc_lib.erl\", line: 329]}\n          ]},\n         {LoggerHandlerKit.Plug, :call,\n          [\n            %Plug.Conn{},\n            %{test_pid: #PID<0.238.0>}\n          ]}}, []}\n     ],\n     label: {:error_logger, :error_msg},\n     format: ~c\"Ranch listener ~p, connection process ~p, stream ~p had its request process ~p exit with reason ~0p~n\",\n     elixir_translation: [\n       \"#PID<0.455.0>\",\n       \" running \",\n       \"LoggerHandlerKit.Plug\",\n       [\" (connection \", \"#PID<0.454.0>\", \", stream id \", \"1\", 41],\n       \" terminated\n\",\n       [\n         [\"Server: \", \"localhost\", \":\", \"8001\", 32, 40, \"http\", 41, 10],\n         [\"Request: \", \"GET\", 32, \"/exception\", 10]\n       ] |\n       \"** (exit) an exception was raised:\n    ** (RuntimeError) oops\n        (logger_handler_kit 0.2.0) lib/logger_handler_kit/plug.ex:8: anonymous fn/2 in LoggerHandlerKit.Plug.do_match/4\n        (logger_handler_kit 0.2.0) deps/plug/lib/plug/router.ex:246: anonymous fn/4 in LoggerHandlerKit.Plug.dispatch/2\n        (telemetry 1.3.0) /Users/user/projects/logger_handler_kit/deps/telemetry/src/telemetry.erl:324: :telemetry.span/3\n        (logger_handler_kit 0.2.0) deps/plug/lib/plug/router.ex:242: LoggerHandlerKit.Plug.dispatch/2\n        (logger_handler_kit 0.2.0) lib/logger_handler_kit/plug.ex:1: LoggerHandlerKit.Plug.plug_builder_call/2\n        (plug_cowboy 2.7.3) lib/plug/cowboy/handler.ex:11: Plug.Cowboy.Handler.init/2\n        (cowboy 2.13.0) /Users/user/projects/logger_handler_kit/deps/cowboy/src/cowboy_handler.erl:37: :cowboy_handler.execute/2\n        (cowboy 2.13.0) /Users/user/projects/logger_handler_kit/deps/cowboy/src/cowboy_stream_h.erl:310: :cowboy_stream_h.execute/3\n        (cowboy 2.13.0) /Users/user/projects/logger_handler_kit/deps/cowboy/src/cowboy_stream_h.erl:299: :cowboy_stream_h.request_process/3\n        (stdlib 6.2.2) proc_lib.erl:329: :proc_lib.init_p_do_apply/3\"\n     ]\n   }},\n  level: :error\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#plug_error/3-example-log-event-cowboy-elixir-1-19"},{"type":"function","title":"LoggerHandlerKit.Act.proc_lib_crash/1","doc":"Unlike bare process crashes, `:proc_lib` crashes are reported as reports and with plenty of metadata.\n\n`:proc_lib` is an Erlang module used for working with processes [_that adhere to\nOTP design principles_](https://www.erlang.org/doc/apps/stdlib/proc_lib.html).\n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#proc_lib_crash/1"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.proc_lib_crash/1","doc":"```elixir\ntest \":proc_lib crash exception\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.proc_lib_crash(:exception)\n  LoggerHandlerKit.Assert.assert_logged(ref)\n\n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#proc_lib_crash/1-example-test"},{"type":"function","title":"Example Log Event (< 1.19) - LoggerHandlerKit.Act.proc_lib_crash/1","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error_report, type: :crash_report},\n    initial_call: {LoggerHandlerKit.Act, :\"-proc_lib_crash/0-fun-0-\", 0},\n    line: 948,\n    pid: #PID<0.321.0>,\n    time: 1745103540970106,\n    file: ~c\"proc_lib.erl\",\n    gl: #PID<0.69.0>,\n    domain: [:otp, :sasl],\n    logger_formatter: %{title: ~c\"CRASH REPORT\"},\n    mfa: {:proc_lib, :crash_report, 4},\n    report_cb: &:proc_lib.report_cb/2,\n    ancestors: [#PID<0.312.0>],\n    callers: [#PID<0.312.0>],\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-proc_lib_crash/0-fun-0-\", 1,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 1350,\n          error_info: %{module: Exception}\n        ]},\n       {:proc_lib, :init_p, 3, [file: ~c\"proc_lib.erl\", line: 313]}\n     ]}\n  },\n  msg: {:string,\n   [\n     \"Process \",\n     \"#PID<0.321.0>\",\n     \" terminating\",\n     [\n       10,\n       \"** (RuntimeError) oops\",\n       [\"\n    \" |\n        \"(logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:1350: anonymous fn/1 in LoggerHandlerKit.Act.proc_lib_crash/0\"],\n       [\"\n    \" | \"(stdlib 6.2.1) proc_lib.erl:313: :proc_lib.init_p/3\"]\n     ],\n     [\n       ~c\"\n\",\n       \"Initial Call: \",\n       \"anonymous fn/0 in LoggerHandlerKit.Act.proc_lib_crash/0\",\n       ~c\"\n\",\n       \"Ancestors: \",\n       \"[#PID<0.312.0>]\",\n       [~c\"\n\", \"Message Queue Length\", 58, 32, \"0\"],\n       [~c\"\n\", \"Messages\", 58, 32, \"[]\"],\n       [~c\"\n\", \"Links\", 58, 32, \"[]\"],\n       [~c\"\n\", \"Dictionary\", 58, 32, \"[\"$callers\": [#PID<0.312.0>]]\"],\n       [~c\"\n\", \"Trapping Exits\", 58, 32, \"false\"],\n       [~c\"\n\", \"Status\", 58, 32, \":running\"],\n       [~c\"\n\", \"Heap Size\", 58, 32, \"376\"],\n       [~c\"\n\", \"Stack Size\", 58, 32, \"29\"],\n       [~c\"\n\", \"Reductions\", 58, 32, \"423\"]\n     ],\n     []\n   ]},\n  level: :error\n}\n```","ref":"LoggerHandlerKit.Act.html#proc_lib_crash/1-example-log-event-1-19"},{"type":"function","title":"Example Log Event (1.19+) - LoggerHandlerKit.Act.proc_lib_crash/1","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error_report, type: :crash_report},\n    initial_call: {LoggerHandlerKit.Act, :\"-proc_lib_crash/0-fun-0-\", 0},\n    line: 948,\n    pid: #PID<0.215.0>,\n    time: 1745103479744344,\n    file: ~c\"proc_lib.erl\",\n    gl: #PID<0.69.0>,\n    domain: [:otp, :sasl],\n    logger_formatter: %{title: ~c\"CRASH REPORT\"},\n    mfa: {:proc_lib, :crash_report, 4},\n    report_cb: &Logger.Utils.translated_cb/1,\n    ancestors: [#PID<0.204.0>],\n    callers: [#PID<0.204.0>],\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-proc_lib_crash/0-fun-0-\", 1,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 1355,\n          error_info: %{module: Exception}\n        ]},\n       {:proc_lib, :init_p, 3, [file: ~c\"proc_lib.erl\", line: 313]}\n     ]}\n  },\n  msg: {:report,\n   %{\n     label: {:proc_lib, :crash},\n     report: [\n       [\n         initial_call: {LoggerHandlerKit.Act, :\"-proc_lib_crash/0-fun-0-\", []},\n         pid: #PID<0.215.0>,\n         registered_name: [],\n         process_label: :undefined,\n         error_info: {:error, %RuntimeError{message: \"oops\"},\n          [\n            {LoggerHandlerKit.Act, :\"-proc_lib_crash/0-fun-0-\", 1,\n             [\n               file: ~c\"lib/logger_handler_kit/act.ex\",\n               line: 1355,\n               error_info: %{module: Exception}\n             ]},\n            {:proc_lib, :init_p, 3, [file: ~c\"proc_lib.erl\", line: 313]}\n          ]},\n         ancestors: [#PID<0.204.0>],\n         message_queue_len: 0,\n         messages: [],\n         links: [],\n         dictionary: [\"$callers\": [#PID<0.204.0>]],\n         trap_exit: false,\n         status: :running,\n         heap_size: 376,\n         stack_size: 29,\n         reductions: 429\n       ],\n       []\n     ],\n     elixir_translation: [...]\n   }},\n  level: :error\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#proc_lib_crash/1-example-log-event-1-19"},{"type":"function","title":"LoggerHandlerKit.Act.string_message/0","doc":"The most basic and perhaps most common log message is a simple [string](`String`) passed to one of the `Logger` functions:\n\n```elixir\nLogger.info(\"Hello World\")\n```\n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#string_message/0"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.string_message/0","doc":"```elixir\ntest \"string message\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.string_message()\n  LoggerHandlerKit.Assert.assert_logged(ref)\n  \n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#string_message/0-example-test"},{"type":"function","title":"Example Log Event - LoggerHandlerKit.Act.string_message/0","doc":"```elixir\n%{\n  meta: %{\n    pid: #PID<0.218.0>,\n    time: 1744679377402910,\n    gl: #PID<0.69.0>,\n    domain: [:elixir]\n  },\n  msg: {:string, \"Hello World\"},\n  level: :info\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#string_message/0-example-log-event"},{"type":"function","title":"LoggerHandlerKit.Act.struct_report/0","doc":"Structs are technically maps, but they [do not inherit any of the protocols](`e:elixir:structs.html#structs-are-bare-maps-underneath`)\n\nFamously, the lack of protocol implementation is what makes it hard to dump structs to JSON.\n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#struct_report/0"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.struct_report/0","doc":"```elixir\ntest \"struct report\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.struct_report()\n  LoggerHandlerKit.Assert.assert_logged(ref)\n\n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#struct_report/0-example-test"},{"type":"function","title":"Example Log Event - LoggerHandlerKit.Act.struct_report/0","doc":"```elixir\n%{\n  meta: %{\n    pid: #PID<0.204.0>,\n    time: 1744768094816278,\n    gl: #PID<0.69.0>,\n    domain: [:elixir]\n  },\n  msg: {:report, %LoggerHandlerKit.FakeStruct{hello: \"world\"}},\n  level: :info\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#struct_report/0-example-log-event"},{"type":"function","title":"LoggerHandlerKit.Act.supervisor_progress_report/1","doc":"`Supervisor` module emits quite a few log reports, which are gated behind the \n`handle_sasl_reports` configuration option. They come in different log levels as \nwell!\n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#supervisor_progress_report/1"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.supervisor_progress_report/1","doc":"```elixir\ntest \"supervisor progress report failed to start child\", %{handler_ref: ref, io_ref: io_ref} do\n  LoggerHandlerKit.Act.supervisor_progress_report(:failed_to_start_child)\n  LoggerHandlerKit.Assert.assert_logged(ref)\n\n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#supervisor_progress_report/1-example-test"},{"type":"function","title":"Example Log Event (< 1.19) - LoggerHandlerKit.Act.supervisor_progress_report/1","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{\n      tag: :error_report,\n      type: :supervisor_report,\n      report_cb: &:supervisor.format_log/1\n    },\n    line: 937,\n    pid: #PID<0.200.0>,\n    time: 1745204329480468,\n    file: ~c\"supervisor.erl\",\n    gl: #PID<0.69.0>,\n    domain: [:otp, :sasl],\n    logger_formatter: %{title: ~c\"SUPERVISOR REPORT\"},\n    mfa: {:supervisor, :start_children, 2},\n    report_cb: &:supervisor.format_log/2\n  },\n  msg: {:string,\n   [\n     \"Child \",\n     \":task\",\n     \" of Supervisor \",\n     [\"#PID<0.200.0>\", \" (\", \"Supervisor.Default\", 41],\n     32,\n     \"failed to start\",\n     \"\n** (exit) \",\n     \":reason\",\n     \"\nStart Call: \",\n     \"LoggerHandlerKit.Helper.run(#Function<26.21775298/0 in LoggerHandlerKit.Act.supervisor_progress_report/1>)\",\n     [\"\nRestart: \" | \":permanent\"],\n     [],\n     [\"\nShutdown: \" | \"5000\"],\n     [\"\nType: \" | \":worker\"]\n   ]},\n  level: :error\n}\n```","ref":"LoggerHandlerKit.Act.html#supervisor_progress_report/1-example-log-event-1-19"},{"type":"function","title":"Example Log Event (1.19+) - LoggerHandlerKit.Act.supervisor_progress_report/1","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{\n      tag: :error_report,\n      type: :supervisor_report,\n      report_cb: &:supervisor.format_log/1\n    },\n    line: 937,\n    pid: #PID<0.326.0>,\n    time: 1745204485120228,\n    file: ~c\"supervisor.erl\",\n    gl: #PID<0.69.0>,\n    domain: [:otp, :sasl],\n    logger_formatter: %{title: ~c\"SUPERVISOR REPORT\"},\n    mfa: {:supervisor, :start_children, 2},\n    report_cb: &Logger.Utils.translated_cb/1\n  },\n  msg: {:report,\n   %{\n     label: {:supervisor, :start_error},\n     report: [\n       supervisor: {#PID<0.326.0>, Supervisor.Default},\n       errorContext: :start_error,\n       reason: :reason,\n       offender: [\n         pid: :undefined,\n         id: :task,\n         mfargs: {LoggerHandlerKit.Helper, :run,\n          [#Function<26.30156712/0 in LoggerHandlerKit.Act.supervisor_progress_report/1>]},\n         restart_type: :permanent,\n         significant: false,\n         shutdown: 5000,\n         child_type: :worker\n       ]\n     ],\n     elixir_translation: [...]\n   }},\n  level: :error\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#supervisor_progress_report/1-example-log-event-1-19"},{"type":"function","title":"LoggerHandlerKit.Act.task_error/1","doc":"`Task`s are a popular way to run asynchronous workloads in Elixir applications.\n\nTasks are special in that they break some rules. First of all, technically Task\nerrors are reports and include a special `report_cb` callback as per the [Erlang \nconvention](https://www.erlang.org/doc/apps/kernel/logger.html#t:report_cb/0).\nHowever, usually this callback is not offered a chance to shine as Task errors\nare translated by `Logger.Translator`. This means that Elixir defines 2 ways of \nformatting `Task` reports: in the `Task` module itself, and in the \n`Logger.Translator`. In the majority of cases we see the output of the latter. \nThis is not particularly important for handlers, but it's a fun fact nevertheless.\n\n<!-- tabs-open -->","ref":"LoggerHandlerKit.Act.html#task_error/1"},{"type":"function","title":"Example Test - LoggerHandlerKit.Act.task_error/1","doc":"```elixir\ntest \"task error exception\", %{handler_ref: ref} do\n  LoggerHandlerKit.Act.task_error(:exception)\n  LoggerHandlerKit.Assert.assert_logged(ref)\n\n  # handler-specific assertions\nend\n```","ref":"LoggerHandlerKit.Act.html#task_error/1-example-test"},{"type":"function","title":"Example Log Event (< 1.19) - LoggerHandlerKit.Act.task_error/1","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error_msg},\n    pid: #PID<0.321.0>,\n    time: 1744770005614313,\n    gl: #PID<0.69.0>,\n    domain: [:otp, :elixir],\n    report_cb: &Task.Supervised.format_report/1,\n    callers: [#PID<0.312.0>],\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-task_error/1-fun-0-\", 0,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 671,\n          error_info: %{module: Exception}\n        ]},\n       {Task.Supervised, :invoke_mfa, 2,\n        [file: ~c\"lib/task/supervised.ex\", line: 101]}\n     ]}\n  },\n  msg: {:string,\n   [\n     \"Task #PID<0.321.0> started from #PID<0.312.0> terminating\",\n     [\n       [10 | \"** (RuntimeError) oops\"],\n       [\"\n    \" |\n        \"(logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:671: anonymous fn/0 in LoggerHandlerKit.Act.task_error/1\"],\n       [\"\n    \" |\n        \"(elixir 1.18.3) lib/task/supervised.ex:101: Task.Supervised.invoke_mfa/2\"]\n     ],\n     \"\nFunction: #Function<9.52854244/0 in LoggerHandlerKit.Act.task_error/1>\",\n     \"\n    Args: []\"\n   ]},\n  level: :error\n}\n```","ref":"LoggerHandlerKit.Act.html#task_error/1-example-log-event-1-19"},{"type":"function","title":"Example Log Event (1.19+) - LoggerHandlerKit.Act.task_error/1","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error_msg},\n    pid: #PID<0.202.0>,\n    time: 1744769872184986,\n    gl: #PID<0.69.0>,\n    domain: [:otp, :elixir],\n    report_cb: &Logger.Utils.translated_cb/1,\n    callers: [#PID<0.191.0>],\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-task_error/1-fun-0-\", 0,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 621,\n          error_info: %{module: Exception}\n        ]},\n       {Task.Supervised, :invoke_mfa, 2,\n        [file: ~c\"lib/task/supervised.ex\", line: 105]}\n     ]}\n  },\n  msg: {:report,\n   %{\n     label: {Task.Supervisor, :terminating},\n     report: %{\n       args: [],\n       function: #Function<9.61584632/0 in LoggerHandlerKit.Act.task_error/1>,\n       name: #PID<0.202.0>,\n       reason: {%RuntimeError{message: \"oops\"},\n        [\n          {LoggerHandlerKit.Act, :\"-task_error/1-fun-0-\", 0,\n           [\n             file: ~c\"lib/logger_handler_kit/act.ex\",\n             line: 621,\n             error_info: %{module: Exception}\n           ]},\n          {Task.Supervised, :invoke_mfa, 2,\n           [file: ~c\"lib/task/supervised.ex\", line: 105]}\n        ]},\n       process_label: :undefined,\n       starter: #PID<0.191.0>\n     },\n     elixir_translation: [ ... ]\n   }},\n  level: :error\n}\n```\n\n<!-- tabs-close -->","ref":"LoggerHandlerKit.Act.html#task_error/1-example-log-event-1-19"},{"type":"module","title":"LoggerHandlerKit.Arrange","doc":"Functions that help set up logger handler tests.","ref":"LoggerHandlerKit.Arrange.html"},{"type":"function","title":"LoggerHandlerKit.Arrange.add_handler/4","doc":"Attaches a logger handler with individual translation and ownership filters.","ref":"LoggerHandlerKit.Arrange.html#add_handler/4"},{"type":"function","title":"Arguments - LoggerHandlerKit.Arrange.add_handler/4","doc":"* `handler_id` is an id that will be used for handler. A good idea is to use a test name or a test module.\n\n* `handler_module` a handler module that we want to attach. For example, `:logger_std_h` or `Sentry.LoggerHandler`.\n\n* `config` this is a _handler config_. The one that `handler_module` defines. A small one, of `:term()` type.\n\n* `big_config_override` is a map that will be merged into _handler config_. But a different one, \nthe one of `:logger_handler.config()` type. This is also the place to put \n`handle_otp_reports` and `handle_sasl_reports` options, they will be passed to the \ntranslator.","ref":"LoggerHandlerKit.Arrange.html#add_handler/4-arguments"},{"type":"function","title":"Return - LoggerHandlerKit.Arrange.add_handler/4","doc":"The function returns a two element tuple. The first element is a map that contains \n`handler_id` and `handler_ref`. It can be merged into the test context. The second \nelement is an anonymous function that detaches the handler. Drop it into \n`ExUnit.Callbacks.on_exit/1` callback.\n\nThe `handler_module` is not attached as-is. Instead, it is wrapped in `LoggerHandlerKit.HandlerWrapper`. \nEvery time a handlers' `c::logger_handler.log/2` callback is invoked, it sends a message to the \ntest process which can be received with `LoggerHandlerKit.Assert.assert_logged/1` function.","ref":"LoggerHandlerKit.Arrange.html#add_handler/4-return"},{"type":"function","title":"Example - LoggerHandlerKit.Arrange.add_handler/4","doc":"```elixir\nsetup %{test: test} = context do\n  {context, on_exit} =\n    LoggerHandlerKit.Arrange.add_handler(\n      test,\n      :logger_std_h,\n      %{},\n      %{formatter: Logger.default_formatter(), level: :debug}\n    )\n\n  on_exit(on_exit)\n  context\nend\n```","ref":"LoggerHandlerKit.Arrange.html#add_handler/4-example"},{"type":"function","title":"LoggerHandlerKit.Arrange.allow/3","doc":"Give a particular process access to a logger handler attached by `add_handler/4`.\n\nIt shouldn't be necessary when using `LoggerHandlerKit.Act` functions, but for custom cases you might need it.\n\n> #### Remember! {: .tip}\n>\n> In a lot of cases, [caller tracking](`m:Task#module-ancestor-and-caller-tracking`) is enough to automatically propagate ownership information. Use it!","ref":"LoggerHandlerKit.Arrange.html#allow/3"},{"type":"function","title":"LoggerHandlerKit.Arrange.ensure_per_handler_translation/1","doc":"Replace global logger translator with per-handler ones, so that each test can configure it independently.\n\nNormally, `handle_otp_reports` and `handle_sasl_reports` are global configuration\noptions, and changing them in tests is sufficient to make the tests sync.\nHowever, there is a workaround! In reality, both options are read at startup\nand passed to the `logger_translator` filter, which Elixir Logger attaches as a\nprimary filter. We can detach this primary filter and reattach it\nto each logger handler independently. This way, each handler can take advantage of\ndifferent translator configurations.\n\nThis function is designed to be run as a test setup, and it does exactly that. It detaches\nthe global translator filter and attaches it to each existing handler.\n\n```elixir\ndefmodule LoggerHandlerKit.DefaultLoggerTest do\n  use ExUnit.Case, async: true\n\n  setup_all {LoggerHandlerKit.Arrange, :ensure_per_handler_translation}\n  ...\nend\n```\n\n> #### Exception {: .warning}\n>\n> Not all handlers will get a personal translator. Attaching logger translator to a logger handler can sometimes lead to\n> surprising results because of the way logger filters work. If a handler has `filter_default: :stop` in configuration, it\n> effectively drops all events by default and expects filters to explicitly tell which events they want through. Logger\n> Translator follows a different paradigm, it blocks things that it wants to be blocked and happily allows everything else.\n> When paired, a conservative handler and a underzelous filter will result in handler receiving events it didn't expect. To\n> mitigate this, we only attach translator to handlers with `filter_default: :log`","ref":"LoggerHandlerKit.Arrange.html#ensure_per_handler_translation/1"},{"type":"function","title":"LoggerHandlerKit.Arrange.ownership_filter/2","doc":"Ownership filter drops all events that do not have access to the logger handler attached by `add_handler/4`. This is one of the main things that makes async tests possible.\n\nOwnership filter is built with `NimbleOwnership` mechanism, the same that powers `Mox`, \n`Req`, etc. For the most part, caller tracking is enough to correctly propagate \nownership information across processes. However, in some cases the logging process is \ncompletely detached from the originating process. In this case, ownership filter will \ncheck `pid` key in metadata, and if _that_ pid has access to the handler, it will \nallow it and also ask `Mox` to allow it, if present.","ref":"LoggerHandlerKit.Arrange.html#ownership_filter/2"},{"type":"module","title":"LoggerHandlerKit.Assert","doc":"Helpful assertions.","ref":"LoggerHandlerKit.Assert.html"},{"type":"function","title":"LoggerHandlerKit.Assert.assert_logged/1","doc":"Receives a message from handler under test using `handler_ref` generated by\n`LoggerHandlerKit.Arrange.add_handler/4`.\n\nThis assertion does 2 important things:\n1. It acts as a synchronization point. Once the message is received, we know that the\n   logger handler is done processing the log event.\n2. It properly reraises any error that the logger handler could encounter. It's sometimes\n   hard to get an error message when testing error messages, so surfacing errors to\n   ExUnit is important.","ref":"LoggerHandlerKit.Assert.html#assert_logged/1"},{"type":"function","title":"Example - LoggerHandlerKit.Assert.assert_logged/1","doc":"```elixir\ntest \"string message\"do\n  {%{handler_ref: ref}, on_exit} =\n    LoggerHandlerKit.Arrange.add_handler(test, :logger_std_h, %{},\n   %{formatter: Logger.default_formatter()}\n  )\n\n  on_exit(on_exit)\n\n  LoggerHandlerKit.Act.string_message()\n  LoggerHandlerKit.Assert.assert_logged(ref)\nend\n```","ref":"LoggerHandlerKit.Assert.html#assert_logged/1-example"},{"type":"module","title":"LoggerHandlerKit.HandlerWrapper","doc":"Logger Handler which wraps another handler to make it more testable.\n\nLogger handlers can be annoying to test for a number of reasons:\n\n1. If they encounter an error, they are detached instead of crashing, which\nmakes it harder to understand what's going on, as the test failure will not\nreport the actual error.\n2. Once OTP gets involved, logger handlers might find\nthemselves executing in exotic parts of the system. This makes it difficult\nfor tests to know when to proceed with assertions.\n\n`HandlerWrapper` expects the following configuration keys:\n* `test_pid` – PID of a test process.\n* `ref` – a reference to accompany messages it will send to the test process.\n* `handler_module` – a handler module to wrap.\n* `inside_config` – a config to pass to the wrapped handler.\n\nWhenever the `log/2` callback is invoked, the wrapper handler will pass the\nlog event to the wrapped module, catch any error if needed, and once finished,\nsend a `{ref, :log_call_completed}` or `{ref, {:handler_error, {kind, reason,\n__STACKTRACE__}}}` message to the test process.\n\n`HandlerWrapper` is a fairly low level part of the toolbox, so before using\nit, check out `LoggerHandlerKit.Arrange.add_handler/4` and\n`LoggerHandlerKit.Assert.assert_logged/1`.","ref":"LoggerHandlerKit.HandlerWrapper.html"},{"type":"module","title":"Example - LoggerHandlerKit.HandlerWrapper","doc":"```elixir\niex(2)> ref = make_ref()\n#Reference<0.3804490001.995622913.186390>\niex(3)>     :logger.add_handler(:wrapped_handler, LoggerHandlerKit.HandlerWrapper, %{\n...(3)>       config: %{test_pid: self(), ref: ref, handler_module: :logger_std_h, inside_config: %{}},\n...(3)>       formatter: Logger.default_formatter()\n...(3)>     })\n:ok\niex(4)> Logger.info(\"Hello!\")\n\n18:27:38.096 [info] Hello!\n\n18:27:38.096 [info] Hello!\n:ok\niex(5)> receive do msg -> msg end\n{#Reference<0.3804490001.995622913.186390>, :log_call_completed}\n```","ref":"LoggerHandlerKit.HandlerWrapper.html#module-example"},{"type":"module","title":"LoggerHandlerKit.Plug","doc":"","ref":"LoggerHandlerKit.Plug.html"},{"type":"function","title":"LoggerHandlerKit.Plug.call/2","doc":"","ref":"LoggerHandlerKit.Plug.html#call/2"},{"type":"function","title":"LoggerHandlerKit.Plug.init/1","doc":"","ref":"LoggerHandlerKit.Plug.html#init/1"},{"type":"module","title":"LoggerHandlerKit.FakeStruct","doc":"A module defining a struct with one field. If you need _any_ struct, why not this one?","ref":"LoggerHandlerKit.FakeStruct.html"},{"type":"module","title":"LoggerHandlerKit.GenServer","doc":"A very simple GenServer module which only purpose is to crash in a way that tests want it too.\n\nIs also propagates `$callers` to facilitate proper caller tracking.\n\nFor usage examples check `LoggerHandlerKit.Act.genserver_crash/1`.","ref":"LoggerHandlerKit.GenServer.html"},{"type":"function","title":"LoggerHandlerKit.GenServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"LoggerHandlerKit.GenServer.html#child_spec/1"},{"type":"function","title":"LoggerHandlerKit.GenServer.start/2","doc":"","ref":"LoggerHandlerKit.GenServer.html#start/2"},{"type":"module","title":"LoggerHandlerKit.GenStatem","doc":"A very simple GenStatem module which only purpose is to crash in a way that tests want it too.\n\nIs also propagates `$callers` to facilitate proper caller tracking.\n\nFor usage examples check `LoggerHandlerKit.Act.gen_statem_crash/1`.","ref":"LoggerHandlerKit.GenStatem.html"},{"type":"function","title":"LoggerHandlerKit.GenStatem.start/2","doc":"","ref":"LoggerHandlerKit.GenStatem.html#start/2"},{"type":"function","title":"LoggerHandlerKit.GenStatem.started/3","doc":"","ref":"LoggerHandlerKit.GenStatem.html#started/3"},{"type":"module","title":"LoggerHandlerKit.Helper","doc":"","ref":"LoggerHandlerKit.Helper.html"},{"type":"function","title":"LoggerHandlerKit.Helper.run/1","doc":"A helper function for when we want to run an anonymous function, but the interface\nrequires a MFA tuple.","ref":"LoggerHandlerKit.Helper.html#run/1"},{"type":"extras","title":"LoggerHandlerKit","doc":"# LoggerHandlerKit\n\n📚 Logger Handler Kit is an _educational_ Hex package! The goal is to help people\nunderstand how to write and test logger handlers. This is a great package if you\nwant to:\n\n* learn more about logging\n* write your own logger handler\n* bootstrap tests\n* improve your test coverage\n* learn how to make your tests async\n\nYou can use this package in a number of ways:\n\n* Read through the docs to learn about concepts\n* Read through the code to learn how to make things work\n* Add as a test dependency for your app and actually use test helpers","ref":"readme.html"},{"type":"extras","title":"AAA - LoggerHandlerKit","doc":"Since the package's primary goal is to aid with tests, its most important parts\nare organized into three modules that embody what is called an AAA test pattern. The\nAAA stands for three main stages of a test: Arrange, Act and Assert:\n\n* `LoggerHandlerKit.Arrange` helps with setting up handlers for tests\n* `LoggerHandlerKit.Act` suggests a menu of interesting test cases\n* `LoggerHandlerKit.Assert` provides a single assert function with a very important job","ref":"readme.html#aaa"},{"type":"extras","title":"Example Test Suite - LoggerHandlerKit","doc":"Logger Handler Kit comes with a fully asynchronous test suite for the default\nElixir logger handler, inspired by tests in `Sentry`, `DiscoLog` and Elixir\nitself. It demonstrates how a handler can be tested with a fully async suite by\nemploying pretty much every trick in LoggerHandlerKit's repertoire:\n\n1. It creates a dedicated handler for each test. \n2. It guards the test handler against irrelevant log events with an ownership filter. \n3. It configures the `logger_std_h` handler to write to a fake IO device instead of\nstdout. The device relays all received writes back to the test process. \n4. It uses `LoggerHandlerKit.Arrange.ensure_per_handler_translation/1` to enable\nswitching `handle_otp_reports` and `handle_sasl_reports` without impacting the rest\nof the application.\n\nHere's a visualization of how test setup compares to the regular application\nsetup. Note how test process logs use a separate \"path\" thanks to a dedicated\nlogger handler guarded by the ownership filter:\n\n\n```mermaid\n---\nconfig:\n    look: handDrawn\n    theme: neutral\n---\nflowchart TD\n    subgraph before[Normal Setup]\n        test_pid[Test Process] --> test_log\n        other_tests[Other Tests] --> other_log \n        test_log@{shape: rounded, label: [log event]} --> translator\n        other_log@{shape: rounded, label: [log event]} --> translator\n        \n        subgraph primary[Primary Filters]\n            translator[:logger_translator]\n        end\n        \n        subgraph handlers\n            default_handler\n        end\n        \n        translator --> default_handler[:default handler]\n        default_handler --> stdout[standard output]\n    end\n    subgraph after[Async Setup]\n        test_pid2[Test Process] --> test_log2\n        other_tests2[Other Tests] --> other_log2 \n        test_log2@{shape: rounded, label: [log event]} --> no_filters\n        other_log2@{shape: rounded, label: [log event]} --> no_filters\n        subgraph primary2[Primary Filters]\n            no_filters@{shape: braces, label: [No Filters]}\n        end\n        no_filters --> default_translator\n        no_filters --> ownership_filter\n        subgraph handlers2[Handlers]\n            default_translator[:logger_translator] --> default_handler2[:default handler]\n            ownership_filter[ownership filter] --> test_translator[:logger_translator]\n            test_translator[:logger_translator] --> test_handler[test handler]\n        end\n        default_handler2 --> stdout2[standard output]\n        test_handler[test handler] --> fakeio[FakeIODevice]\n    end\n```","ref":"readme.html#example-test-suite"},{"type":"extras","title":"Getting ~~Started~~ Good - LoggerHandlerKit","doc":"1. Add the package to your project as a test dependency\n\n```elixir\ndef deps do\n  [\n    # The package only makes sense for tests!\n    {:logger_handler_kit, only: :test, \"~> 0.3.0\"}\n  ]\nend\n```\n\n2. Replace `MyHandler` with your handler name and make this test green:\n\n```elixir\ndefmodule MyHandlerTest do\n  use ExUnit.Case, async: true\n\n  setup_all {LoggerHandlerKit.Arrange, :ensure_per_handler_translation}\n\n  setup %{test: test} = context do\n    {context, on_exit} =\n      LoggerHandlerKit.Arrange.add_handler(\n        test,\n        MyHandler,\n        %{}\n      )\n\n    on_exit(on_exit)\n    context\n  end\n\n  test \"string message\", %{handler_ref: ref} do\n    LoggerHandlerKit.Act.string_message()\n    LoggerHandlerKit.Assert.assert_logged(ref)\n  end\nend\n```\n\n3. Add assertions that make sense for your handler to the test and make it green again.\n4. Write a passing test for each function in the `LoggerHandlerKit.Act` module.\n\n🎉 Congratulations, your logger handler is now pretty good and you have a decent asynchronous test suite on your hands.","ref":"readme.html#getting-started-good"},{"type":"extras","title":"TODO - LoggerHandlerKit","doc":"* [x] cover metadata\n* [ ] cover overload protection\n* [x] cover encoding/serialization\n* [ ] cover logging packages overview","ref":"readme.html#todo"},{"type":"extras","title":"Log Translation","doc":"# Log Translation\n\nElixir and Erlang are different languages, but Elixir runs on the Erlang\nplatform and it doesn't take much time until the platform starts to manifest\nitself. A lot of common errors originate in Erlang and if printed as-is, look\nforeign in the context of Elixir applications. To avoid this, Elixir performs\nsomething called log translation. This is one of the main functions of `Logger`\napplications Compare these two GenServer errors, with and without translation:\n\n<!-- tabs-open -->","ref":"translation.html"},{"type":"extras","title":"With Translation - Log Translation","doc":"```text\n13:43:32.538 [error] GenServer #PID<0.198.0> terminating\n** (RuntimeError) oops\n    (logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:528: anonymous fn/0 in LoggerHandlerKit.Act.genserver_crash/1\n    (stdlib 6.2.2) gen_server.erl:2381: :gen_server.try_handle_call/4\n    (stdlib 6.2.2) gen_server.erl:2410: :gen_server.handle_msg/6\n    (stdlib 6.2.2) proc_lib.erl:329: :proc_lib.init_p_do_apply/3\nLast message (from #PID<0.197.0>): {:run, #Function<16.413966/0 in LoggerHandlerKit.Act.genserver_crash/1>}\nState: nil\nClient #PID<0.197.0> is alive\n\n    (stdlib 6.2.2) gen.erl:260: :gen.do_call/4\n    (elixir 1.19.0-dev) lib/gen_server.ex:1139: GenServer.call/3\n    (logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:528: LoggerHandlerKit.Act.genserver_crash/1\n    (elixir 1.19.0-dev) src/elixir.erl:363: :elixir.eval_external_handler/3\n    (stdlib 6.2.2) erl_eval.erl:919: :erl_eval.do_apply/7\n    (elixir 1.19.0-dev) src/elixir.erl:341: :elixir.eval_forms/4\n    (elixir 1.19.0-dev) lib/module/parallel_checker.ex:141: Module.ParallelChecker.verify/1\n    (iex 1.19.0-dev) lib/iex/evaluator.ex:340: IEx.Evaluator.eval_and_inspect/3\n```","ref":"translation.html#with-translation"},{"type":"extras","title":"Without Translation - Log Translation","doc":"```text\n13:44:53.011 [error] ** Generic server <0.199.0> terminating\n** Last message in was {run,#Fun }\n** When Server state == nil\n** Reason for termination ==\n** {#{message => <<\"oops\">>,'__struct__' => 'Elixir.RuntimeError',\n      '__exception__' => true},\n    [{'Elixir.LoggerHandlerKit.Act','-genserver_crash/1-fun-0-',0,\n         [{file,\"lib/logger_handler_kit/act.ex\"},\n          {line,528},\n          {error_info,#{module => 'Elixir.Exception'}}]},\n     {gen_server,try_handle_call,4,[{file,\"gen_server.erl\"},{line,2381}]},\n     {gen_server,handle_msg,6,[{file,\"gen_server.erl\"},{line,2410}]},\n     {proc_lib,init_p_do_apply,3,[{file,\"proc_lib.erl\"},{line,329}]}]}\n** Client <0.197.0> stacktrace\n** [{gen,do_call,4,[{file,\"gen.erl\"},{line,260}]},\n    {'Elixir.GenServer',call,3,[{file,\"lib/gen_server.ex\"},{line,1139}]},\n    {'Elixir.LoggerHandlerKit.Act',genserver_crash,1,\n                                   [{file,\"lib/logger_handler_kit/act.ex\"},\n                                    {line,528}]},\n    {elixir,eval_external_handler,3,[{file,\"src/elixir.erl\"},{line,363}]},\n    {erl_eval,do_apply,7,[{file,\"erl_eval.erl\"},{line,919}]},\n    {elixir,eval_forms,4,[{file,\"src/elixir.erl\"},{line,341}]},\n    {'Elixir.Module.ParallelChecker',verify,1,\n                                     [{file,\"lib/module/parallel_checker.ex\"},\n                                      {line,141}]},\n    {'Elixir.IEx.Evaluator',eval_and_inspect,3,\n                            [{file,\"lib/iex/evaluator.ex\"},{line,340}]}]\n\n\n13:44:53.016 [error]   crasher:\n    initial call: 'Elixir.LoggerHandlerKit.GenServer':init/1\n    pid: <0.199.0>\n    registered_name: []\n    exception error: #Elixir.RuntimeError\n      in function  'Elixir.LoggerHandlerKit.Act':'-genserver_crash/1-fun-0-'/0 (lib/logger_handler_kit/act.ex, line 528)\n         *** oops\n      in call from gen_server:try_handle_call/4 (gen_server.erl, line 2381)\n      in call from gen_server:handle_msg/6 (gen_server.erl, line 2410)\n    ancestors: [<0.197.0>,<0.98.0>]\n    message_queue_len: 0\n    messages: []\n    links: []\n    dictionary: [{'$callers',[<0.197.0>]}]\n    trap_exit: false\n    status: running\n    heap_size: 6772\n    stack_size: 29\n    reductions: 19158\n  neighbours: []\n```\n\n<!-- tabs-close -->\n\nIt's very clear how important translation is for people who are fluent primarily\nin Elixir.","ref":"translation.html#without-translation"},{"type":"extras","title":"Translation is a Filter - Log Translation","doc":"Log translation is implemented as a primary filter, which means it will be\ncalled before handlers and any handler-specific filters. Remember that filters\nand logger configuration in general can be altered by users, although it's\nsomewhat uncommon to remove or modify the `logger_translator` specifically.\n\nTo better visualize this, here is where the translation happens:\n\n```elixir\niex(1)> :logger.get_primary_config()\n%{\n  level: :debug,\n  filter_default: :log,\n  filters: [\n    # Here it is! Translation filter\n    logger_translator: {&Logger.Utils.translator/2,\n     %{otp: true, sasl: false, translators: [{Logger.Translator, :translate}]}},\n    logger_process_level: {&Logger.Utils.process_level/2, []}\n  ],\n  metadata: %{}\n}\n```","ref":"translation.html#translation-is-a-filter"},{"type":"extras","title":"1.18 vs 1.19+ - Log Translation","doc":"Before Elixir 1.19, log translation used to _replace_ the original Erlang\nstructured report with an unstructured log message. If a handler wanted to access\nsome of the data in the original report, it had to parse the log message.\nStarting from Elixir 1.19, translation is _inserted into_ the report, thus\nmaking the original report accessible to the handlers. Here is an example:\n\n<!-- tabs-open -->","ref":"translation.html#1-18-vs-1-19"},{"type":"extras","title":"Before Translation - Log Translation","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error_report, type: :crash_report},\n    line: 950,\n    pid: #PID<0.203.0>,\n    time: 1746910751187661,\n    file: ~c\"proc_lib.erl\",\n    gl: #PID<0.70.0>,\n    domain: [:otp, :sasl],\n    logger_formatter: %{title: ~c\"CRASH REPORT\"},\n    mfa: {:proc_lib, :crash_report, 4},\n    report_cb: &:proc_lib.report_cb/2\n  },\n  msg: {:report,\n   %{\n     label: {:proc_lib, :crash},\n     report: [\n       [\n         initial_call: {LoggerHandlerKit.GenServer, :init, [:Argument__1]},\n         pid: #PID<0.203.0>,\n         registered_name: [],\n         process_label: :undefined,\n         error_info: {:error, %RuntimeError{message: \"oops\"},\n          [\n            {LoggerHandlerKit.Act, :\"-genserver_crash/1-fun-0-\", 0,\n             [\n               file: ~c\"lib/logger_handler_kit/act.ex\",\n               line: 528,\n               error_info: %{module: Exception}\n             ]},\n            {:gen_server, :try_handle_call, 4,\n             [file: ~c\"gen_server.erl\", line: 2381]},\n            {:gen_server, :handle_msg, 6,\n             [file: ~c\"gen_server.erl\", line: 2410]},\n            {:proc_lib, :init_p_do_apply, 3,\n             [file: ~c\"proc_lib.erl\", line: 329]}\n          ]},\n         ancestors: [#PID<0.197.0>, #PID<0.98.0>],\n         message_queue_len: 0,\n         messages: [],\n         links: [],\n         dictionary: [\"$callers\": [#PID<0.197.0>]],\n         trap_exit: false,\n         status: :running,\n         heap_size: 1598,\n         stack_size: 29,\n         reductions: 28455\n       ],\n       []\n     ]\n   }},\n  level: :error\n}\n```","ref":"translation.html#before-translation"},{"type":"extras","title":"1.18 Translation - Log Translation","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error, report_cb: &:gen_server.format_log/1},\n    line: 2646,\n    pid: #PID<0.1105.0>,\n    time: 1746911049539570,\n    file: ~c\"gen_server.erl\",\n    gl: #PID<0.70.0>,\n    domain: [:otp],\n    report_cb: &:gen_server.format_log/2,\n    mfa: {:gen_server, :error_info, 8},\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-genserver_crash/1-fun-0-\", 0,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 528,\n          error_info: %{module: Exception}\n        ]},\n       {:gen_server, :try_handle_call, 4,\n        [file: ~c\"gen_server.erl\", line: 2381]},\n       {:gen_server, :handle_msg, 6, [file: ~c\"gen_server.erl\", line: 2410]},\n       {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n     ]}\n  },\n  msg: {:string,\n   [\n     [\n       \"GenServer \",\n       \"#PID<0.1105.0>\",\n       \" terminating\",\n       [\n         [10 | \"** (RuntimeError) oops\"],\n         [\"\\n    \" |\n          \"(logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:528: anonymous fn/0 in LoggerHandlerKit.Act.genserver_crash/1\"],\n         [\"\\n    \" |\n          \"(stdlib 6.2.2) gen_server.erl:2381: :gen_server.try_handle_call/4\"],\n         [\"\\n    \" |\n          \"(stdlib 6.2.2) gen_server.erl:2410: :gen_server.handle_msg/6\"],\n         [\"\\n    \" |\n          \"(stdlib 6.2.2) proc_lib.erl:329: :proc_lib.init_p_do_apply/3\"]\n       ],\n       [],\n       \"\\nLast message\",\n       [\" (from \", \"#PID<0.1099.0>\", \")\"],\n       \": \",\n       \"{:run, #Function<16.413966/0 in LoggerHandlerKit.Act.genserver_crash/1>}\"\n     ],\n     \"\\nState: \",\n     \"nil\",\n     \"\\nClient \",\n     \"#PID<0.1099.0>\",\n     \" is alive\\n\",\n     [\"\\n    \" | \"(stdlib 6.2.2) gen.erl:260: :gen.do_call/4\"],\n     [\"\\n    \" | \"(elixir 1.18.3) lib/gen_server.ex:1125: GenServer.call/3\"],\n     [\"\\n    \" |\n      \"(logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:528: LoggerHandlerKit.Act.genserver_crash/1\"],\n     [\"\\n    \" |\n      \"(elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\"],\n     [\"\\n    \" | \"(stdlib 6.2.2) erl_eval.erl:919: :erl_eval.do_apply/7\"],\n     [\"\\n    \" | \"(elixir 1.18.3) src/elixir.erl:364: :elixir.eval_forms/4\"],\n     [\"\\n    \" |\n      \"(elixir 1.18.3) lib/module/parallel_checker.ex:120: Module.ParallelChecker.verify/1\"],\n     [\"\\n    \" |\n      \"(iex 1.18.3) lib/iex/evaluator.ex:336: IEx.Evaluator.eval_and_inspect/3\"]\n   ]},\n  level: :error\n}\n```","ref":"translation.html#1-18-translation"},{"type":"extras","title":"1.19 Translation - Log Translation","doc":"```elixir\n%{\n  meta: %{\n    error_logger: %{tag: :error, report_cb: &:gen_server.format_log/1},\n    line: 2646,\n    pid: #PID<0.205.0>,\n    time: 1746910905177484,\n    file: ~c\"gen_server.erl\",\n    gl: #PID<0.70.0>,\n    domain: [:otp],\n    mfa: {:gen_server, :error_info, 8},\n    report_cb: &Logger.Utils.translated_cb/1,\n    crash_reason: {%RuntimeError{message: \"oops\"},\n     [\n       {LoggerHandlerKit.Act, :\"-genserver_crash/1-fun-0-\", 0,\n        [\n          file: ~c\"lib/logger_handler_kit/act.ex\",\n          line: 528,\n          error_info: %{module: Exception}\n        ]},\n       {:gen_server, :try_handle_call, 4,\n        [file: ~c\"gen_server.erl\", line: 2381]},\n       {:gen_server, :handle_msg, 6, [file: ~c\"gen_server.erl\", line: 2410]},\n       {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n     ]}\n  },\n  msg: {:report,\n   %{\n     label: {:gen_server, :terminate},\n     name: #PID<0.205.0>,\n     reason: {%RuntimeError{message: \"oops\"},\n      [\n        {LoggerHandlerKit.Act, :\"-genserver_crash/1-fun-0-\", 0,\n         [\n           file: ~c\"lib/logger_handler_kit/act.ex\",\n           line: 528,\n           error_info: %{module: Exception}\n         ]},\n        {:gen_server, :try_handle_call, 4,\n         [file: ~c\"gen_server.erl\", line: 2381]},\n        {:gen_server, :handle_msg, 6, [file: ~c\"gen_server.erl\", line: 2410]},\n        {:proc_lib, :init_p_do_apply, 3, [file: ~c\"proc_lib.erl\", line: 329]}\n      ]},\n     log: [],\n     state: nil,\n     process_label: :undefined,\n     last_message: {:run,\n      #Function<16.413966/0 in LoggerHandlerKit.Act.genserver_crash/1>},\n     client_info: {#PID<0.197.0>,\n      {#PID<0.197.0>,\n       [\n         {:gen, :do_call, 4, [file: ~c\"gen.erl\", line: 260]},\n         {GenServer, :call, 3, [file: ~c\"lib/gen_server.ex\", line: 1139]},\n         {LoggerHandlerKit.Act, :genserver_crash, 1,\n          [file: ~c\"lib/logger_handler_kit/act.ex\", line: 528]},\n         {:elixir, :eval_external_handler, 3,\n          [file: ~c\"src/elixir.erl\", line: 363]},\n         {:erl_eval, :do_apply, 7, [file: ~c\"erl_eval.erl\", line: 919]},\n         {:elixir, :eval_forms, 4, [file: ~c\"src/elixir.erl\", line: 341]},\n         {Module.ParallelChecker, :verify, 1,\n          [file: ~c\"lib/module/parallel_checker.ex\", line: 141]},\n         {IEx.Evaluator, :eval_and_inspect, 3,\n          [file: ~c\"lib/iex/evaluator.ex\", line: 340]}\n       ]}},\n     elixir_translation: [\n       [\n         \"GenServer \",\n         \"#PID<0.205.0>\",\n         \" terminating\",\n         [\n           [10 | \"** (RuntimeError) oops\"],\n           [\"\\n    \" |\n            \"(logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:528: anonymous fn/0 in LoggerHandlerKit.Act.genserver_crash/1\"],\n           [\"\\n    \" |\n            \"(stdlib 6.2.2) gen_server.erl:2381: :gen_server.try_handle_call/4\"],\n           [\"\\n    \" |\n            \"(stdlib 6.2.2) gen_server.erl:2410: :gen_server.handle_msg/6\"],\n           [\"\\n    \" |\n            \"(stdlib 6.2.2) proc_lib.erl:329: :proc_lib.init_p_do_apply/3\"]\n         ],\n         [],\n         \"\\nLast message\",\n         [\" (from \", \"#PID<0.197.0>\", \")\"],\n         \": \",\n         \"{:run, #Function<16.413966/0 in LoggerHandlerKit.Act.genserver_crash/1>}\"\n       ],\n       \"\\nState: \",\n       \"nil\",\n       \"\\nClient \",\n       \"#PID<0.197.0>\",\n       \" is alive\\n\",\n       [\"\\n    \" | \"(stdlib 6.2.2) gen.erl:260: :gen.do_call/4\"],\n       [\"\\n    \" |\n        \"(elixir 1.19.0-dev) lib/gen_server.ex:1139: GenServer.call/3\"],\n       [\"\\n    \" |\n        \"(logger_handler_kit 0.1.0) lib/logger_handler_kit/act.ex:528: LoggerHandlerKit.Act.genserver_crash/1\"],\n       [\"\\n    \" |\n        \"(elixir 1.19.0-dev) src/elixir.erl:363: :elixir.eval_external_handler/3\"],\n       [\"\\n    \" | \"(stdlib 6.2.2) erl_eval.erl:919: :erl_eval.do_apply/7\"],\n       [\"\\n    \" |\n        \"(elixir 1.19.0-dev) src/elixir.erl:341: :elixir.eval_forms/4\"],\n       [\"\\n    \" |\n        \"(elixir 1.19.0-dev) lib/module/parallel_checker.ex:141: Module.ParallelChecker.verify/1\"],\n       [\"\\n    \" |\n        \"(iex 1.19.0-dev) lib/iex/evaluator.ex:340: IEx.Evaluator.eval_and_inspect/3\"]\n     ]\n   }},\n  level: :error\n}\n```\n\n<!-- tabs-close -->","ref":"translation.html#1-19-translation"},{"type":"extras","title":"More Than Translation - Log Translation","doc":"","ref":"translation.html#more-than-translation"},{"type":"extras","title":"Enforcing `handle_*_reports` configuration options - Log Translation","doc":"In addition to translating logs, the `logger_translator` filter is also responsible\nfor enforcing the `handle_otp_reports` and `handle_sasl_reports` [configuration\noptions](https://hexdocs.pm/logger/Logger.html#module-boot-configuration). This\nis worth keeping in mind and may come up during debugging. Trying to understand\nwhich part of the system dropped an OTP log event may lead to the surprising\ndiscovery that the logger translator did it.","ref":"translation.html#enforcing-handle_-_reports-configuration-options"},{"type":"extras","title":"Populating `crash_reason` - Log Translation","doc":"Another important job of `Logger.Translator` is [populating the `crash_reason`\nmetadata](unhandled.md). This metadata key contains a `{reason, stacktrace}` tuple and serves\nas an Elixir convention for marking log events that result from unhandled\nexceptions, throws, or abnormal exits.","ref":"translation.html#populating-crash_reason"},{"type":"extras","title":"Unhandled Errors","doc":"# Unhandled Errors\n\nLogging services commonly distinguish between handled and unhandled errors. \nUnhandled errors are considered unexpected and receive special treatment: they \nare fingerprinted, aggregated, tracked for resolution, and, most importantly,\ntrigger user notifications.\n\nElixir does not have an explicit concept of an unhandled error. From the logger\nperspective, an unhandled error looks like a log event and goes through the same\npipeline as other log events. Every logging service must therefore come up with its\nown definition of unhandled errors and ways to identify them. Fortunately, there is\none useful trick for this.\n\n## `crash_reason` metadata\n\nEven though Elixir doesn't have a strict definition for which logs can be\nconsidered unhandled, it helpfully puts a special [`crash_reason` key](`Logger#module-metadata`) into\nlog metadata for events that either resulted in or could result in some sort of \nprocess crash. This is done by `Logger.Translator` and is possible because\n`Logger.Translator` holds a catalogue of common error reports and knows when to\nenrich them with `crash_reason`. Thanks, `Logger.Translator`!\n\nLooking for `crash_reason` is the simplest way to know if an error is unhandled.\nHowever, be aware that `crash_reason` can be set manually by application code.\nIn this case, it isn't _truly_ unhandled. Still, if somebody explicitly _made it\nlook unhandled_, there's really no reason to treat it differently.\n\nBelow is an example showing how to report a handled error with `crash_reason`:\n\n```elixir\ndefmodule Example do\n  require Logger\n\n  def handle_all(fun) do\n    try do\n      fun.()\n    catch\n      kind, reason ->\n        stacktrace = __STACKTRACE__\n        normalized = Exception.normalize(kind, reason, stacktrace)\n        reason = if kind == :throw, do: {:nocatch, reason}, else: normalized\n        \n        Exception.format(kind, normalized, stacktrace)\n        |> Logger.error(crash_reason: {reason, stacktrace}) \n    end\n  end\nend\n```\n\n<!-- tabs-open -->","ref":"unhandled.html"},{"type":"extras","title":"Exception - Unhandled Errors","doc":"```elixir\niex> Example.handle_all(fn -> raise \"foo\" end)\n12:32:24.703 [error] ** (RuntimeError) foo\n    iex:3: (file)\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\n    logger.exs:6: Example.handle_all/1\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\n    (stdlib 6.2.2) erl_eval.erl:919: :erl_eval.do_apply/7\n    (elixir 1.18.3) src/elixir.erl:364: :elixir.eval_forms/4\n    (elixir 1.18.3) lib/module/parallel_checker.ex:120: Module.ParallelChecker.verify/1\n    (iex 1.18.3) lib/iex/evaluator.ex:336: IEx.Evaluator.eval_and_inspect/3\n\n:ok\n```\n\nLog event:\n\n```elixir\n%{\n  meta: %{\n    line: 12,\n    pid: #PID<0.117.0>,\n    time: 1748547144703394,\n    file: ~c\"/Users/user/example.exs\",\n    gl: #PID<0.70.0>,\n    domain: [:elixir],\n    mfa: {Example, :handle_all, 1},\n    crash_reason: {%RuntimeError{message: \"foo\"},\n     [\n       {:elixir_eval, :__FILE__, 1, [file: ~c\"iex\", line: 3]},\n       {:elixir, :eval_external_handler, 3,\n        [file: ~c\"src/elixir.erl\", line: 386, error_info: %{module: Exception}]},\n       {Example, :handle_all, 1, [file: ~c\"logger.exs\", line: 6]},\n       {:elixir, :eval_external_handler, 3,\n        [file: ~c\"src/elixir.erl\", line: 386]},\n       {:erl_eval, :do_apply, 7, [file: ~c\"erl_eval.erl\", line: 919]},\n       {:elixir, :eval_forms, 4, [file: ~c\"src/elixir.erl\", line: 364]},\n       {Module.ParallelChecker, :verify, 1,\n        [file: ~c\"lib/module/parallel_checker.ex\", line: 120]},\n       {IEx.Evaluator, :eval_and_inspect, 3,\n        [file: ~c\"lib/iex/evaluator.ex\", line: 336]}\n     ]}\n  },\n  msg: {:string,\n   \"** (RuntimeError) foo\\n    iex:3: (file)\\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\\n    logger.exs:6: Example.handle_all/1\\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\\n    (stdlib 6.2.2) erl_eval.erl:919: :erl_eval.do_apply/7\\n    (elixir 1.18.3) src/elixir.erl:364: :elixir.eval_forms/4\\n    (elixir 1.18.3) lib/module/parallel_checker.ex:120: Module.ParallelChecker.verify/1\\n    (iex 1.18.3) lib/iex/evaluator.ex:336: IEx.Evaluator.eval_and_inspect/3\\n\"},\n  level: :error\n}\n```","ref":"unhandled.html#exception"},{"type":"extras","title":"Throw - Unhandled Errors","doc":"```elixir\niex> Example.handle_all(fn -> throw \"catch!\" end)\n12:37:38.382 [error] ** (throw) \"catch!\"\n    iex:4: (file)\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\n    logger.exs:6: Example.handle_all/1\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\n    (stdlib 6.2.2) erl_eval.erl:919: :erl_eval.do_apply/7\n    (elixir 1.18.3) src/elixir.erl:364: :elixir.eval_forms/4\n    (elixir 1.18.3) lib/module/parallel_checker.ex:120: Module.ParallelChecker.verify/1\n    (iex 1.18.3) lib/iex/evaluator.ex:336: IEx.Evaluator.eval_and_inspect/3\n\n:ok\n```\n\nLog event:\n\n```elixir\n%{\n  meta: %{\n    line: 14,\n    pid: #PID<0.117.0>,\n    time: 1748558079155346,\n    file: ~c\"/Users/user/example.exs\",\n    gl: #PID<0.70.0>,\n    domain: [:elixir],\n    mfa: {Example, :handle_all, 1},\n    crash_reason: {{:nocatch, \"catch!\"},\n     [\n       {:elixir_eval, :__FILE__, 1, [file: ~c\"iex\", line: 6]},\n       {:elixir, :eval_external_handler, 3,\n        [file: ~c\"src/elixir.erl\", line: 386]},\n       {Example, :handle_all, 1, [file: ~c\"logger.exs\", line: 6]},\n       {:elixir, :eval_external_handler, 3,\n        [file: ~c\"src/elixir.erl\", line: 386]},\n       {:erl_eval, :do_apply, 7, [file: ~c\"erl_eval.erl\", line: 919]},\n       {:elixir, :eval_forms, 4, [file: ~c\"src/elixir.erl\", line: 364]},\n       {Module.ParallelChecker, :verify, 1,\n        [file: ~c\"lib/module/parallel_checker.ex\", line: 120]},\n       {IEx.Evaluator, :eval_and_inspect, 3,\n        [file: ~c\"lib/iex/evaluator.ex\", line: 336]}\n     ]}\n  },\n  msg: {:string,\n   \"** (throw) \\\"catch!\\\"\\n    iex:6: (file)\\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\\n    logger.exs:6: Example.handle_all/1\\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\\n    (stdlib 6.2.2) erl_eval.erl:919: :erl_eval.do_apply/7\\n    (elixir 1.18.3) src/elixir.erl:364: :elixir.eval_forms/4\\n    (elixir 1.18.3) lib/module/parallel_checker.ex:120: Module.ParallelChecker.verify/1\\n    (iex 1.18.3) lib/iex/evaluator.ex:336: IEx.Evaluator.eval_and_inspect/3\\n\"},\n  level: :error\n}\n```","ref":"unhandled.html#throw"},{"type":"extras","title":"Exit - Unhandled Errors","doc":"```elixir\niex> Example.handle_all(fn -> exit(\"i quit\") end)\n12:38:42.709 [error] ** (exit) \"i quit\"\n    iex:5: (file)\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\n    logger.exs:6: Example.handle_all/1\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\n    (stdlib 6.2.2) erl_eval.erl:919: :erl_eval.do_apply/7\n    (elixir 1.18.3) src/elixir.erl:364: :elixir.eval_forms/4\n    (elixir 1.18.3) lib/module/parallel_checker.ex:120: Module.ParallelChecker.verify/1\n    (iex 1.18.3) lib/iex/evaluator.ex:336: IEx.Evaluator.eval_and_inspect/3\n\n:ok\n```\n\nLog event:\n\n```elixir\n%{\n  meta: %{\n    line: 12,\n    pid: #PID<0.117.0>,\n    time: 1748547522709480,\n    file: ~c\"/Users/user/example.exs\",\n    gl: #PID<0.70.0>,\n    domain: [:elixir],\n    mfa: {Example, :handle_all, 1},\n    crash_reason: {\"i quit\",\n     [\n       {:elixir_eval, :__FILE__, 1, [file: ~c\"iex\", line: 5]},\n       {:elixir, :eval_external_handler, 3,\n        [file: ~c\"src/elixir.erl\", line: 386]},\n       {Example, :handle_all, 1, [file: ~c\"logger.exs\", line: 6]},\n       {:elixir, :eval_external_handler, 3,\n        [file: ~c\"src/elixir.erl\", line: 386]},\n       {:erl_eval, :do_apply, 7, [file: ~c\"erl_eval.erl\", line: 919]},\n       {:elixir, :eval_forms, 4, [file: ~c\"src/elixir.erl\", line: 364]},\n       {Module.ParallelChecker, :verify, 1,\n        [file: ~c\"lib/module/parallel_checker.ex\", line: 120]},\n       {IEx.Evaluator, :eval_and_inspect, 3,\n        [file: ~c\"lib/iex/evaluator.ex\", line: 336]}\n     ]}\n  },\n  msg: {:string,\n   \"** (exit) \\\"i quit\\\"\\n    iex:5: (file)\\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\\n    logger.exs:6: Example.handle_all/1\\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\\n    (stdlib 6.2.2) erl_eval.erl:919: :erl_eval.do_apply/7\\n    (elixir 1.18.3) src/elixir.erl:364: :elixir.eval_forms/4\\n    (elixir 1.18.3) lib/module/parallel_checker.ex:120: Module.ParallelChecker.verify/1\\n    (iex 1.18.3) lib/iex/evaluator.ex:336: IEx.Evaluator.eval_and_inspect/3\\n\"},\n  level: :error\n}\n```\n\n<!-- tabs-close -->\n\nNone of those errors actually crashed anything, but we deliberately logged them\nas if they did. This is often done by libraries when they want to report errors\nin user code, but cannot let it crash their process.\n[`Bandit`](https://github.com/mtrudel/bandit/blob/b8bf2bc76c1f49885fed5c2e68111e55b9e84e25/lib/bandit/pipeline.ex#L242-L243)\nis a good example. If you are a library author and want to handle errors that\noriginate in the client's code without crashing the process, make sure to be a\ngood citizen and put `crash_reason` into your log message!\n\n> #### `{:nocatch, term}` {: .tip}\n>\n> The convention requires the throw value to be wrapped in a `{:nocatch, term}`\n> tuple. This helps distinguish it from the exit reason.","ref":"unhandled.html#exit"},{"type":"extras","title":"Metadata","doc":"# Metadata\n\n[Logger metadata](`Logger#module-metadata`) is structured data attached to log\nmessages. Metadata can be passed to `Logger` with the message:\n\n```elixir\nLogger.info(\"Hello\", user_id: 123)\n```\n\nor set beforehand with `Logger.metadata/1`:\n\n```elixir\nLogger.metadata(user_id: 123)\n```\n\n> #### Not Just Keyword List {: .tip}\n>\n> Big Logger don't want you to know, but maps totally work as metadata too:\n>\n> ```elixir\n> iex> Logger.metadata(%{hello: \"world\"})\n> :ok\n> iex> Logger.metadata()\n> [hello: \"world\"]\n> ```\n\nMetadata is tricky to use correctly because it behaves very differently in\ndevelopment versus production environments. In development, the default console\nlogger outputs minimal metadata, and even when configured to show more, console\nspace is limited, so developers are pushed to embed important data directly in\nlog messages. Production logging solutions, however, very much prefer structured\nmetadata for filtering and searching, paired with static log messages that\nenable effective fingerprinting and grouping of similar events.\n\nThis guide focuses on the latter approach: using static log messages paired with\nrich metadata:\n\n```elixir\nLogger.error(\"Unexpected API response\", status_code: 422, user_id: 123)\n```\n\nWhen working with metadata, logging libraries typically grapple with two key\nchallenges: serialization and scrubbing.","ref":"metadata.html"},{"type":"extras","title":"Serialization - Metadata","doc":"Metadata can hold Elixir terms of any type, but to send them somewhere and\ndisplay them to users, they must be serialized. Unfortunately, there's no\nuniversally good way to handle this! Elixir's default\n`Logger.Formatter#module-metadata` supports only a handful of types. The\nde-facto expectation, however, is that specialized logging libraries can handle\nany term and display it reasonably well. Consequently, every logging library\nimplements a step where it makes the hard decisions about what to do with\ntuples, structs, and other complex data types. This process is sometimes called\nencoding or sanitization.\n\nOne solution that works well and can be easily integrated into your project is\nthe\n[`LoggerJSON.Formatters.RedactorEncoder.encode/2`](https://hexdocs.pm/logger_json/LoggerJSON.Formatter.RedactorEncoder.html#encode/2)\nfunction. It accepts any Elixir term and makes it JSON-serializable:\n\n```elixir\niex> LoggerJSON.Formatter.RedactorEncoder.encode(%{tuple: {:ok, \"foo\"}, pid: self()}, [])\n%{pid: \"#PID<0.219.0>\", tuple: [:ok, \"foo\"]}\n```","ref":"metadata.html#serialization"},{"type":"extras","title":"Scrubbing - Metadata","doc":"Scrubbing is the process of removing sensitive fields from metadata. Data like\npasswords, API keys, or credit card numbers should never be sent to your logging\nservice unnecessarily. While many logging services implement scrubbing on the\nreceiving end, some libraries handle this on the client side as well.\n\nThe challenge with scrubbing is that it must be configurable. Applications store\ndiverse types of secrets, and no set of default rules can catch them all.\nFortunately, the same solution used for serialization works here too.\n[`LoggerJSON.Formatters.RedactorEncoder.encode/2`](https://hexdocs.pm/logger_json/LoggerJSON.Formatter.RedactorEncoder.html#encode/2)\naccepts a list of \"redactors\" that will be called to scrub potentially sensitive\ndata. It includes a powerful\n[`LoggerJSON.Redactors.RedactKeys`](https://hexdocs.pm/logger_json/LoggerJSON.Redactors.RedactKeys.html)\nredactor that redacts all values stored under specified keys:\n\n```elixir\niex> LoggerJSON.Formatter.RedactorEncoder.encode(\n  %{user: \"Marion\", password: \"SCP-3125\"},\n  [{LoggerJSON.Redactors.RedactKeys, [\"password\"]}]\n)\n%{user: \"Marion\", password: \"[REDACTED]\"}\n```","ref":"metadata.html#scrubbing"},{"type":"extras","title":"Conclusion - Metadata","doc":"While [`LoggerJSON`](https://hex.pm/packages/logger_json)'s primary goal isn't to solve our metadata struggles, it\ncomes with a set of tools that can be very handy. Even if you don't want to\ndepend on it directly, it can provide you with a good starting point for your\nown solution. And `LoggerHandlerKit.Act.metadata_serialization/1` can help you\nwith test cases!","ref":"metadata.html#conclusion"},{"type":"extras","title":"Plug Integration","doc":"# Plug Integration\n\n`Plug` integration is the most common \"addon\" for\n[logging](https://hexdocs.pm/sentry/setup-with-plug-and-phoenix.html)\n[libraries](https://hexdocs.pm/honeybadger/Honeybadger.Plug.html)\n[by](https://github.com/appsignal/appsignal-elixir-plug)\n[far](https://hexdocs.pm/error_tracker/ErrorTracker.Integrations.Plug.html).\nThere are two reasons for this.","ref":"plug-integration.html"},{"type":"extras","title":"1. Metadata - Plug Integration","doc":"`Plug.Conn` is a treasure trove of useful metadata that makes understanding\nerrors much easier. Even basic information like request path and user-agent can\nmake a huge difference in debugging. To take advantage of this rich context and\nsave developers effort, logging libraries typically provide a plug module that\nyou can drop into your router. This plug automatically extracts relevant\nconnection data and adds it to logger metadata. Any errors logged later in the\nsame process will inherit this context.","ref":"plug-integration.html#1-metadata"},{"type":"extras","title":"2. Error Handling - Plug Integration","doc":"Setting metadata from `conn` is straightforward, why do a lot of Plug\nintegration modules consist largely of macros?\n\nFor a long time, `Cowboy` was _the_ web server for Elixir applications, and it\nhas one significant quirk: Cowboy logs errors from a different process than the\none that actually handled the request. Remember our earlier point?\n\n> Any errors logged later in the same process will inherit this context.\n\nWell, for Cowboy errors, all the lovely metadata will be gone. That's why Plug\nintegration modules use macros: they wrap plug execution in a way that allows\nthem to log the error themselves while in the process with access to metadata.\nAnd this is also the reason why virtually every library excludes logs coming\nfrom the `:cowboy` domain - that is, to avoid duplicates.\n\nThese days, custom error handling is hardly necessary. `Bandit` web server is\nthe new default and it executes requests and logs errors in the same process.\nBut the Cowboy situation is better too. `Plug.Cowboy` adapter comes with a\ntranslator that ensures the `conn` struct is available in the log metadata, so a\nlogger handler can extract all the metadata from it again.","ref":"plug-integration.html#2-error-handling"},{"type":"extras","title":"Testing - Plug Integration","doc":"LoggerHandlerKit comes with a `LoggerHandlerKit.Act.plug_error/3` test case to\nhelp you check how plug errors will look when logged by your handler.","ref":"plug-integration.html#testing"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}